
module bindbc.imgui.dynload;

version(BindImGui_Static) {}
else:

import bindbc.loader;
import bindbc.imgui.config,
       bindbc.imgui.bind,
       bindbc.imgui.bind.imgui;

private {
    SharedLib lib;
    ImGuiSupport loadedVersion;
}

void unloadImGui()
{
    if(lib != invalidHandle) {
        lib.unload();
    }
}

ImGuiSupport loadedImGuiVersion() { return loadedVersion; }

bool isImGuiLoaded()
{
    return  lib != invalidHandle;
}

ImGuiSupport loadImGui()
{
    // #1778 prevents me from using static arrays here :(
    version(Windows) {
        const(char)[][2] libNames = [
            "cimgui.dll",
            "cimguid.dll",
        ];
    }
    else version(OSX) {
        const(char)[][1] libNames = [
            "cimgui.dylib"
        ];
    }
    else version(Posix) {
        const(char)[][1] libNames = [
            "cimgui.so"
        ];
    }
    else static assert(0, "bindbc-ImGui is not yet supported on this platform.");

    ImGuiSupport ret;
    foreach(name; libNames) {
        ret = loadImGui(name.ptr);
        if(ret != ImGuiSupport.noLibrary) break;
    }

    return ret;
}

ImGuiSupport loadImGui(const(char)* libName)
{
    lib = load(libName);
    if(lib == invalidHandle) {
        return ImGuiSupport.noLibrary;
    }

    lib.bindSymbol(cast(void**)&ImDrawList_AddCircleFilled, "ImDrawList_AddCircleFilled");
    lib.bindSymbol(cast(void**)&ImGuiPlatformIO_ImGuiPlatformIO, "ImGuiPlatformIO_ImGuiPlatformIO");
    lib.bindSymbol(cast(void**)&igDockContextQueueUndockWindow, "igDockContextQueueUndockWindow");
    lib.bindSymbol(cast(void**)&ImGuiComboPreviewData_ImGuiComboPreviewData, "ImGuiComboPreviewData_ImGuiComboPreviewData");
    lib.bindSymbol(cast(void**)&igEndTable, "igEndTable");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesChineseFull, "ImFontAtlas_GetGlyphRangesChineseFull");
    lib.bindSymbol(cast(void**)&igBringWindowToDisplayFront, "igBringWindowToDisplayFront");
    lib.bindSymbol(cast(void**)&igGetForegroundDrawList_Nil, "igGetForegroundDrawList_Nil");
    lib.bindSymbol(cast(void**)&igGetForegroundDrawList_ViewportPtr, "igGetForegroundDrawList_ViewportPtr");
    lib.bindSymbol(cast(void**)&igGetForegroundDrawList_WindowPtr, "igGetForegroundDrawList_WindowPtr");
    lib.bindSymbol(cast(void**)&igInitialize, "igInitialize");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddCustomRectRegular, "ImFontAtlas_AddCustomRectRegular");
    lib.bindSymbol(cast(void**)&igIsMouseDragPastThreshold, "igIsMouseDragPastThreshold");
    lib.bindSymbol(cast(void**)&igIsGamepadKey, "igIsGamepadKey");
    lib.bindSymbol(cast(void**)&igSetWindowFontScale, "igSetWindowFontScale");
    lib.bindSymbol(cast(void**)&igSliderFloat, "igSliderFloat");
    lib.bindSymbol(cast(void**)&igDestroyPlatformWindows, "igDestroyPlatformWindows");
    lib.bindSymbol(cast(void**)&igImMax, "igImMax");
    lib.bindSymbol(cast(void**)&ImRect_GetTR, "ImRect_GetTR");
    lib.bindSymbol(cast(void**)&igTableSetupColumn, "igTableSetupColumn");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesThai, "ImFontAtlas_GetGlyphRangesThai");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_ClearSelection, "ImGuiInputTextState_ClearSelection");
    lib.bindSymbol(cast(void**)&ImFont_GrowIndex, "ImFont_GrowIndex");
    lib.bindSymbol(cast(void**)&ImGuiListClipper_ForceDisplayRangeByIndices, "ImGuiListClipper_ForceDisplayRangeByIndices");
    lib.bindSymbol(cast(void**)&ImFontAtlas_ClearInputData, "ImFontAtlas_ClearInputData");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddMousePosEvent, "ImGuiIO_AddMousePosEvent");
    lib.bindSymbol(cast(void**)&igClosePopupsOverWindow, "igClosePopupsOverWindow");
    lib.bindSymbol(cast(void**)&ImGuiWindowSettings_destroy, "ImGuiWindowSettings_destroy");
    lib.bindSymbol(cast(void**)&igImBezierCubicCalc, "igImBezierCubicCalc");
    lib.bindSymbol(cast(void**)&igIsMouseDragging, "igIsMouseDragging");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_end, "ImGuiTextBuffer_end");
    lib.bindSymbol(cast(void**)&ImGuiTabBar_destroy, "ImGuiTabBar_destroy");
    lib.bindSymbol(cast(void**)&igDockContextCalcDropPosForDocking, "igDockContextCalcDropPosForDocking");
    lib.bindSymbol(cast(void**)&igLoadIniSettingsFromDisk, "igLoadIniSettingsFromDisk");
    lib.bindSymbol(cast(void**)&igRenderColorRectWithAlphaCheckerboard, "igRenderColorRectWithAlphaCheckerboard");
    lib.bindSymbol(cast(void**)&igFindBestWindowPosForPopup, "igFindBestWindowPosForPopup");
    lib.bindSymbol(cast(void**)&igRadioButton_Bool, "igRadioButton_Bool");
    lib.bindSymbol(cast(void**)&igRadioButton_IntPtr, "igRadioButton_IntPtr");
    lib.bindSymbol(cast(void**)&ImGuiTextFilter_Clear, "ImGuiTextFilter_Clear");
    lib.bindSymbol(cast(void**)&ImRect_TranslateX, "ImRect_TranslateX");
    lib.bindSymbol(cast(void**)&igGetWindowPos, "igGetWindowPos");
    lib.bindSymbol(cast(void**)&ImGuiIO_ClearInputCharacters, "ImGuiIO_ClearInputCharacters");
    lib.bindSymbol(cast(void**)&igImBitArraySetBit, "igImBitArraySetBit");
    lib.bindSymbol(cast(void**)&ImDrawDataBuilder_FlattenIntoSingleLayer, "ImDrawDataBuilder_FlattenIntoSingleLayer");
    lib.bindSymbol(cast(void**)&igRenderTextWrapped, "igRenderTextWrapped");
    lib.bindSymbol(cast(void**)&igImParseFormatSanitizeForPrinting, "igImParseFormatSanitizeForPrinting");
    lib.bindSymbol(cast(void**)&igSpacing, "igSpacing");
    lib.bindSymbol(cast(void**)&igSetClipboardText, "igSetClipboardText");
    lib.bindSymbol(cast(void**)&ImRect_TranslateY, "ImRect_TranslateY");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_c_str, "ImGuiTextBuffer_c_str");
    lib.bindSymbol(cast(void**)&igTabBarFindTabByID, "igTabBarFindTabByID");
    lib.bindSymbol(cast(void**)&igNavMoveRequestSubmit, "igNavMoveRequestSubmit");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_destroy, "ImGuiInputTextState_destroy");
    lib.bindSymbol(cast(void**)&igBeginComboPreview, "igBeginComboPreview");
    lib.bindSymbol(cast(void**)&igGetDrawData, "igGetDrawData");
    lib.bindSymbol(cast(void**)&igPopItemWidth, "igPopItemWidth");
    lib.bindSymbol(cast(void**)&igIsWindowAppearing, "igIsWindowAppearing");
    lib.bindSymbol(cast(void**)&igGetAllocatorFunctions, "igGetAllocatorFunctions");
    lib.bindSymbol(cast(void**)&igRenderRectFilledRangeH, "igRenderRectFilledRangeH");
    lib.bindSymbol(cast(void**)&igSetWindowDock, "igSetWindowDock");
    lib.bindSymbol(cast(void**)&igDataTypeApplyFromText, "igDataTypeApplyFromText");
    lib.bindSymbol(cast(void**)&igFindOrCreateColumns, "igFindOrCreateColumns");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetVoidPtr, "ImGuiStorage_GetVoidPtr");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_GetRedoAvailCount, "ImGuiInputTextState_GetRedoAvailCount");
    lib.bindSymbol(cast(void**)&igIsPopupOpen_Str, "igIsPopupOpen_Str");
    lib.bindSymbol(cast(void**)&igIsPopupOpen_ID, "igIsPopupOpen_ID");
    lib.bindSymbol(cast(void**)&igTableGetSortSpecs, "igTableGetSortSpecs");
    lib.bindSymbol(cast(void**)&igTableDrawBorders, "igTableDrawBorders");
    lib.bindSymbol(cast(void**)&ImGuiTable_ImGuiTable, "ImGuiTable_ImGuiTable");
    lib.bindSymbol(cast(void**)&igInputDouble, "igInputDouble");
    lib.bindSymbol(cast(void**)&igUnindent, "igUnindent");
    lib.bindSymbol(cast(void**)&igImIsFloatAboveGuaranteedIntegerPrecision, "igImIsFloatAboveGuaranteedIntegerPrecision");
    lib.bindSymbol(cast(void**)&igIsDragDropPayloadBeingAccepted, "igIsDragDropPayloadBeingAccepted");
    lib.bindSymbol(cast(void**)&igGetFontSize, "igGetFontSize");
    lib.bindSymbol(cast(void**)&ImGuiMenuColumns_DeclColumns, "ImGuiMenuColumns_DeclColumns");
    lib.bindSymbol(cast(void**)&ImFontAtlas_CalcCustomRectUV, "ImFontAtlas_CalcCustomRectUV");
    lib.bindSymbol(cast(void**)&igGetFrameHeightWithSpacing, "igGetFrameHeightWithSpacing");
    lib.bindSymbol(cast(void**)&ImDrawListSplitter_destroy, "ImDrawListSplitter_destroy");
    lib.bindSymbol(cast(void**)&igGetItemRectMax, "igGetItemRectMax");
    lib.bindSymbol(cast(void**)&igImStreolRange, "igImStreolRange");
    lib.bindSymbol(cast(void**)&igDragInt, "igDragInt");
    lib.bindSymbol(cast(void**)&igGetFont, "igGetFont");
    lib.bindSymbol(cast(void**)&igDragFloatRange2, "igDragFloatRange2");
    lib.bindSymbol(cast(void**)&igTableUpdateLayout, "igTableUpdateLayout");
    lib.bindSymbol(cast(void**)&ImGuiStorage_Clear, "ImGuiStorage_Clear");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_UpdateWorkRect, "ImGuiViewportP_UpdateWorkRect");
    lib.bindSymbol(cast(void**)&igTableNextColumn, "igTableNextColumn");
    lib.bindSymbol(cast(void**)&ImGuiWindow_GetID_Str, "ImGuiWindow_GetID_Str");
    lib.bindSymbol(cast(void**)&ImGuiWindow_GetID_Ptr, "ImGuiWindow_GetID_Ptr");
    lib.bindSymbol(cast(void**)&ImGuiWindow_GetID_Int, "ImGuiWindow_GetID_Int");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildPackCustomRects, "igImFontAtlasBuildPackCustomRects");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_Rect, "ImGuiDockNode_Rect");
    lib.bindSymbol(cast(void**)&igDockBuilderGetNode, "igDockBuilderGetNode");
    lib.bindSymbol(cast(void**)&igIsActiveIdUsingKey, "igIsActiveIdUsingKey");
    lib.bindSymbol(cast(void**)&igIsWindowWithinBeginStackOf, "igIsWindowWithinBeginStackOf");
    lib.bindSymbol(cast(void**)&igTableGetColumnFlags, "igTableGetColumnFlags");
    lib.bindSymbol(cast(void**)&igSetCursorScreenPos, "igSetCursorScreenPos");
    lib.bindSymbol(cast(void**)&igImStristr, "igImStristr");
    lib.bindSymbol(cast(void**)&igSetNextWindowViewport, "igSetNextWindowViewport");
    lib.bindSymbol(cast(void**)&ImFont_GetDebugName, "ImFont_GetDebugName");
    lib.bindSymbol(cast(void**)&igBeginPopupContextWindow, "igBeginPopupContextWindow");
    lib.bindSymbol(cast(void**)&igBeginTable, "igBeginTable");
    lib.bindSymbol(cast(void**)&igButtonEx, "igButtonEx");
    lib.bindSymbol(cast(void**)&igTextEx, "igTextEx");
    lib.bindSymbol(cast(void**)&ImGuiPayload_IsPreview, "ImGuiPayload_IsPreview");
    lib.bindSymbol(cast(void**)&igLabelTextV, "igLabelTextV");
    lib.bindSymbol(cast(void**)&igNavInitRequestApplyResult, "igNavInitRequestApplyResult");
    lib.bindSymbol(cast(void**)&igImStrSkipBlank, "igImStrSkipBlank");
    lib.bindSymbol(cast(void**)&igPushColumnsBackground, "igPushColumnsBackground");
    lib.bindSymbol(cast(void**)&ImGuiWindow_ImGuiWindow, "ImGuiWindow_ImGuiWindow");
    lib.bindSymbol(cast(void**)&igGetScrollMaxX, "igGetScrollMaxX");
    lib.bindSymbol(cast(void**)&ImBitVector_Create, "ImBitVector_Create");
    lib.bindSymbol(cast(void**)&igCloseCurrentPopup, "igCloseCurrentPopup");
    lib.bindSymbol(cast(void**)&igImBitArraySetBitRange, "igImBitArraySetBitRange");
    lib.bindSymbol(cast(void**)&igFindViewportByPlatformHandle, "igFindViewportByPlatformHandle");
    lib.bindSymbol(cast(void**)&ImGuiTableSortSpecs_ImGuiTableSortSpecs, "ImGuiTableSortSpecs_ImGuiTableSortSpecs");
    lib.bindSymbol(cast(void**)&igGetMouseDragDelta, "igGetMouseDragDelta");
    lib.bindSymbol(cast(void**)&igSetWindowCollapsed_Bool, "igSetWindowCollapsed_Bool");
    lib.bindSymbol(cast(void**)&igSetWindowCollapsed_Str, "igSetWindowCollapsed_Str");
    lib.bindSymbol(cast(void**)&igSetWindowCollapsed_WindowPtr, "igSetWindowCollapsed_WindowPtr");
    lib.bindSymbol(cast(void**)&igSplitterBehavior, "igSplitterBehavior");
    lib.bindSymbol(cast(void**)&ImGuiNavItemData_destroy, "ImGuiNavItemData_destroy");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsDockSpace, "ImGuiDockNode_IsDockSpace");
    lib.bindSymbol(cast(void**)&igTableDrawContextMenu, "igTableDrawContextMenu");
    lib.bindSymbol(cast(void**)&igTextDisabled, "igTextDisabled");
    lib.bindSymbol(cast(void**)&igDebugNodeStorage, "igDebugNodeStorage");
    lib.bindSymbol(cast(void**)&igFindBestWindowPosForPopupEx, "igFindBestWindowPosForPopupEx");
    lib.bindSymbol(cast(void**)&igUpdateInputEvents, "igUpdateInputEvents");
    lib.bindSymbol(cast(void**)&igTableSetColumnEnabled, "igTableSetColumnEnabled");
    lib.bindSymbol(cast(void**)&igShowUserGuide, "igShowUserGuide");
    lib.bindSymbol(cast(void**)&igEndPopup, "igEndPopup");
    lib.bindSymbol(cast(void**)&igClearActiveID, "igClearActiveID");
    lib.bindSymbol(cast(void**)&igBeginChildFrame, "igBeginChildFrame");
    lib.bindSymbol(cast(void**)&ImGuiSettingsHandler_destroy, "ImGuiSettingsHandler_destroy");
    lib.bindSymbol(cast(void**)&ImDrawList__ResetForNewFrame, "ImDrawList__ResetForNewFrame");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_append, "ImGuiTextBuffer_append");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddKeyEvent, "ImGuiIO_AddKeyEvent");
    lib.bindSymbol(cast(void**)&igTableGetInstanceData, "igTableGetInstanceData");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_GetUndoAvailCount, "ImGuiInputTextState_GetUndoAvailCount");
    lib.bindSymbol(cast(void**)&igEndFrame, "igEndFrame");
    lib.bindSymbol(cast(void**)&ImGuiTableColumn_destroy, "ImGuiTableColumn_destroy");
    lib.bindSymbol(cast(void**)&ImGuiTextRange_empty, "ImGuiTextRange_empty");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_ClearText, "ImGuiInputTextState_ClearText");
    lib.bindSymbol(cast(void**)&igIsRectVisible_Nil, "igIsRectVisible_Nil");
    lib.bindSymbol(cast(void**)&igIsRectVisible_Vec2, "igIsRectVisible_Vec2");
    lib.bindSymbol(cast(void**)&ImGuiInputTextCallbackData_HasSelection, "ImGuiInputTextCallbackData_HasSelection");
    lib.bindSymbol(cast(void**)&igCalcWrapWidthForPos, "igCalcWrapWidthForPos");
    lib.bindSymbol(cast(void**)&igGetIDWithSeed, "igGetIDWithSeed");
    lib.bindSymbol(cast(void**)&igImUpperPowerOfTwo, "igImUpperPowerOfTwo");
    lib.bindSymbol(cast(void**)&igColorConvertRGBtoHSV, "igColorConvertRGBtoHSV");
    lib.bindSymbol(cast(void**)&igIsMouseClicked, "igIsMouseClicked");
    lib.bindSymbol(cast(void**)&igPushFocusScope, "igPushFocusScope");
    lib.bindSymbol(cast(void**)&igSetNextWindowFocus, "igSetNextWindowFocus");
    lib.bindSymbol(cast(void**)&igGetDefaultFont, "igGetDefaultFont");
    lib.bindSymbol(cast(void**)&igGetClipboardText, "igGetClipboardText");
    lib.bindSymbol(cast(void**)&igIsAnyItemHovered, "igIsAnyItemHovered");
    lib.bindSymbol(cast(void**)&igTableResetSettings, "igTableResetSettings");
    lib.bindSymbol(cast(void**)&ImGuiListClipper_ImGuiListClipper, "ImGuiListClipper_ImGuiListClipper");
    lib.bindSymbol(cast(void**)&igImQsort, "igImQsort");
    lib.bindSymbol(cast(void**)&igTableGetHoveredColumn, "igTableGetHoveredColumn");
    lib.bindSymbol(cast(void**)&igImStrlenW, "igImStrlenW");
    lib.bindSymbol(cast(void**)&igGetWindowDockNode, "igGetWindowDockNode");
    lib.bindSymbol(cast(void**)&igBeginPopup, "igBeginPopup");
    lib.bindSymbol(cast(void**)&ImGuiNavItemData_Clear, "ImGuiNavItemData_Clear");
    lib.bindSymbol(cast(void**)&igTableGetRowIndex, "igTableGetRowIndex");
    lib.bindSymbol(cast(void**)&igImFileGetSize, "igImFileGetSize");
    lib.bindSymbol(cast(void**)&ImGuiSettingsHandler_ImGuiSettingsHandler, "ImGuiSettingsHandler_ImGuiSettingsHandler");
    lib.bindSymbol(cast(void**)&igMenuItem_Bool, "igMenuItem_Bool");
    lib.bindSymbol(cast(void**)&igMenuItem_BoolPtr, "igMenuItem_BoolPtr");
    lib.bindSymbol(cast(void**)&igDockBuilderFinish, "igDockBuilderFinish");
    lib.bindSymbol(cast(void**)&ImGuiStyleMod_ImGuiStyleMod_Int, "ImGuiStyleMod_ImGuiStyleMod_Int");
    lib.bindSymbol(cast(void**)&ImGuiStyleMod_ImGuiStyleMod_Float, "ImGuiStyleMod_ImGuiStyleMod_Float");
    lib.bindSymbol(cast(void**)&ImGuiStyleMod_ImGuiStyleMod_Vec2, "ImGuiStyleMod_ImGuiStyleMod_Vec2");
    lib.bindSymbol(cast(void**)&ImFontConfig_destroy, "ImFontConfig_destroy");
    lib.bindSymbol(cast(void**)&igBeginPopupEx, "igBeginPopupEx");
    lib.bindSymbol(cast(void**)&igImCharIsBlankA, "igImCharIsBlankA");
    lib.bindSymbol(cast(void**)&igImStrTrimBlanks, "igImStrTrimBlanks");
    lib.bindSymbol(cast(void**)&ImGuiListClipper_End, "ImGuiListClipper_End");
    lib.bindSymbol(cast(void**)&igResetMouseDragDelta, "igResetMouseDragDelta");
    lib.bindSymbol(cast(void**)&igDestroyContext, "igDestroyContext");
    lib.bindSymbol(cast(void**)&igSetNextWindowContentSize, "igSetNextWindowContentSize");
    lib.bindSymbol(cast(void**)&igSaveIniSettingsToDisk, "igSaveIniSettingsToDisk");
    lib.bindSymbol(cast(void**)&igGetWindowScrollbarRect, "igGetWindowScrollbarRect");
    lib.bindSymbol(cast(void**)&igBeginComboPopup, "igBeginComboPopup");
    lib.bindSymbol(cast(void**)&igGetKeyName, "igGetKeyName");
    lib.bindSymbol(cast(void**)&igGetTopMostAndVisiblePopupModal, "igGetTopMostAndVisiblePopupModal");
    lib.bindSymbol(cast(void**)&ImGuiTableSettings_GetColumnSettings, "ImGuiTableSettings_GetColumnSettings");
    lib.bindSymbol(cast(void**)&igTableSetupScrollFreeze, "igTableSetupScrollFreeze");
    lib.bindSymbol(cast(void**)&igInputTextMultiline, "igInputTextMultiline");
    lib.bindSymbol(cast(void**)&igIsClippedEx, "igIsClippedEx");
    lib.bindSymbol(cast(void**)&igGetWindowScrollbarID, "igGetWindowScrollbarID");
    lib.bindSymbol(cast(void**)&ImGuiTextFilter_IsActive, "ImGuiTextFilter_IsActive");
    lib.bindSymbol(cast(void**)&ImDrawListSharedData_ImDrawListSharedData, "ImDrawListSharedData_ImDrawListSharedData");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetMouseCursorTexData, "ImFontAtlas_GetMouseCursorTexData");
    lib.bindSymbol(cast(void**)&igImParseFormatSanitizeForScanning, "igImParseFormatSanitizeForScanning");
    lib.bindSymbol(cast(void**)&igGetWindowAlwaysWantOwnTabBar, "igGetWindowAlwaysWantOwnTabBar");
    lib.bindSymbol(cast(void**)&ImGuiTableColumnSettings_ImGuiTableColumnSettings, "ImGuiTableColumnSettings_ImGuiTableColumnSettings");
    lib.bindSymbol(cast(void**)&igBeginDockableDragDropTarget, "igBeginDockableDragDropTarget");
    lib.bindSymbol(cast(void**)&ImGuiPlatformMonitor_destroy, "ImGuiPlatformMonitor_destroy");
    lib.bindSymbol(cast(void**)&igColorEditOptionsPopup, "igColorEditOptionsPopup");
    lib.bindSymbol(cast(void**)&igGetTextLineHeightWithSpacing, "igGetTextLineHeightWithSpacing");
    lib.bindSymbol(cast(void**)&ImGuiTableInstanceData_destroy, "ImGuiTableInstanceData_destroy");
    lib.bindSymbol(cast(void**)&igPushStyleVar_Float, "igPushStyleVar_Float");
    lib.bindSymbol(cast(void**)&igPushStyleVar_Vec2, "igPushStyleVar_Vec2");
    lib.bindSymbol(cast(void**)&igIsActiveIdUsingNavInput, "igIsActiveIdUsingNavInput");
    lib.bindSymbol(cast(void**)&igImStrnicmp, "igImStrnicmp");
    lib.bindSymbol(cast(void**)&igLogText, "igLogText");
    lib.bindSymbol(cast(void**)&igFindHoveredViewportFromPlatformWindowStack, "igFindHoveredViewportFromPlatformWindowStack");
    lib.bindSymbol(cast(void**)&igGetInputTextState, "igGetInputTextState");
    lib.bindSymbol(cast(void**)&igFindRenderedTextEnd, "igFindRenderedTextEnd");
    lib.bindSymbol(cast(void**)&ImFontAtlas_ClearFonts, "ImFontAtlas_ClearFonts");
    lib.bindSymbol(cast(void**)&igTextColoredV, "igTextColoredV");
    lib.bindSymbol(cast(void**)&ImGuiNavItemData_ImGuiNavItemData, "ImGuiNavItemData_ImGuiNavItemData");
    lib.bindSymbol(cast(void**)&igSetActiveIdUsingKey, "igSetActiveIdUsingKey");
    lib.bindSymbol(cast(void**)&igIsKeyReleased, "igIsKeyReleased");
    lib.bindSymbol(cast(void**)&igTabItemLabelAndCloseButton, "igTabItemLabelAndCloseButton");
    lib.bindSymbol(cast(void**)&ImGuiTableColumnSortSpecs_ImGuiTableColumnSortSpecs, "ImGuiTableColumnSortSpecs_ImGuiTableColumnSortSpecs");
    lib.bindSymbol(cast(void**)&igLogToClipboard, "igLogToClipboard");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesKorean, "ImFontAtlas_GetGlyphRangesKorean");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_SetBit, "ImFontGlyphRangesBuilder_SetBit");
    lib.bindSymbol(cast(void**)&igLogSetNextTextDecoration, "igLogSetNextTextDecoration");
    lib.bindSymbol(cast(void**)&igTableFixColumnSortDirection, "igTableFixColumnSortDirection");
    lib.bindSymbol(cast(void**)&igStyleColorsClassic, "igStyleColorsClassic");
    lib.bindSymbol(cast(void**)&ImGuiTabBar_GetTabOrder, "ImGuiTabBar_GetTabOrder");
    lib.bindSymbol(cast(void**)&igBegin, "igBegin");
    lib.bindSymbol(cast(void**)&igButton, "igButton");
    lib.bindSymbol(cast(void**)&igBeginMenuBar, "igBeginMenuBar");
    lib.bindSymbol(cast(void**)&igDataTypeClamp, "igDataTypeClamp");
    lib.bindSymbol(cast(void**)&igRenderText, "igRenderText");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_Clear, "ImFontGlyphRangesBuilder_Clear");
    lib.bindSymbol(cast(void**)&ImGuiMenuColumns_destroy, "ImGuiMenuColumns_destroy");
    lib.bindSymbol(cast(void**)&igImStrncpy, "igImStrncpy");
    lib.bindSymbol(cast(void**)&ImGuiNextWindowData_ImGuiNextWindowData, "ImGuiNextWindowData_ImGuiNextWindowData");
    lib.bindSymbol(cast(void**)&igImBezierCubicClosestPointCasteljau, "igImBezierCubicClosestPointCasteljau");
    lib.bindSymbol(cast(void**)&igItemAdd, "igItemAdd");
    lib.bindSymbol(cast(void**)&igIsWindowNavFocusable, "igIsWindowNavFocusable");
    lib.bindSymbol(cast(void**)&igGetScrollY, "igGetScrollY");
    lib.bindSymbol(cast(void**)&ImGuiOldColumnData_ImGuiOldColumnData, "ImGuiOldColumnData_ImGuiOldColumnData");
    lib.bindSymbol(cast(void**)&ImRect_GetWidth, "ImRect_GetWidth");
    lib.bindSymbol(cast(void**)&igEndListBox, "igEndListBox");
    lib.bindSymbol(cast(void**)&igGetItemStatusFlags, "igGetItemStatusFlags");
    lib.bindSymbol(cast(void**)&igPopFocusScope, "igPopFocusScope");
    lib.bindSymbol(cast(void**)&igGetStyleColorVec4, "igGetStyleColorVec4");
    lib.bindSymbol(cast(void**)&igTableFindByID, "igTableFindByID");
    lib.bindSymbol(cast(void**)&igShutdown, "igShutdown");
    lib.bindSymbol(cast(void**)&igDockBuilderRemoveNodeDockedWindows, "igDockBuilderRemoveNodeDockedWindows");
    lib.bindSymbol(cast(void**)&igTablePushBackgroundChannel, "igTablePushBackgroundChannel");
    lib.bindSymbol(cast(void**)&ImRect_ClipWith, "ImRect_ClipWith");
    lib.bindSymbol(cast(void**)&ImRect_GetTL, "ImRect_GetTL");
    lib.bindSymbol(cast(void**)&ImDrawListSplitter_ImDrawListSplitter, "ImDrawListSplitter_ImDrawListSplitter");
    lib.bindSymbol(cast(void**)&ImDrawList__CalcCircleAutoSegmentCount, "ImDrawList__CalcCircleAutoSegmentCount");
    lib.bindSymbol(cast(void**)&igSetWindowFocus_Nil, "igSetWindowFocus_Nil");
    lib.bindSymbol(cast(void**)&igSetWindowFocus_Str, "igSetWindowFocus_Str");
    lib.bindSymbol(cast(void**)&igInvisibleButton, "igInvisibleButton");
    lib.bindSymbol(cast(void**)&igScaleWindowsInViewport, "igScaleWindowsInViewport");
    lib.bindSymbol(cast(void**)&igRenderMouseCursor, "igRenderMouseCursor");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildInit, "igImFontAtlasBuildInit");
    lib.bindSymbol(cast(void**)&igTextColored, "igTextColored");
    lib.bindSymbol(cast(void**)&igSliderScalar, "igSliderScalar");
    lib.bindSymbol(cast(void**)&igTableSetColumnIndex, "igTableSetColumnIndex");
    lib.bindSymbol(cast(void**)&igRenderPlatformWindowsDefault, "igRenderPlatformWindowsDefault");
    lib.bindSymbol(cast(void**)&ImDrawListSplitter_ClearFreeMemory, "ImDrawListSplitter_ClearFreeMemory");
    lib.bindSymbol(cast(void**)&ImGuiStyle_ImGuiStyle, "ImGuiStyle_ImGuiStyle");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsHiddenTabBar, "ImGuiDockNode_IsHiddenTabBar");
    lib.bindSymbol(cast(void**)&ImGuiOldColumnData_destroy, "ImGuiOldColumnData_destroy");
    lib.bindSymbol(cast(void**)&ImFontConfig_ImFontConfig, "ImFontConfig_ImFontConfig");
    lib.bindSymbol(cast(void**)&igIsMouseDown, "igIsMouseDown");
    lib.bindSymbol(cast(void**)&ImGuiTabBar_GetTabName, "ImGuiTabBar_GetTabName");
    lib.bindSymbol(cast(void**)&igDebugNodeTabBar, "igDebugNodeTabBar");
    lib.bindSymbol(cast(void**)&igNewLine, "igNewLine");
    lib.bindSymbol(cast(void**)&igGetPlatformIO, "igGetPlatformIO");
    lib.bindSymbol(cast(void**)&igMemFree, "igMemFree");
    lib.bindSymbol(cast(void**)&igCalcTypematicRepeatAmount, "igCalcTypematicRepeatAmount");
    lib.bindSymbol(cast(void**)&igNextColumn, "igNextColumn");
    lib.bindSymbol(cast(void**)&igRenderFrame, "igRenderFrame");
    lib.bindSymbol(cast(void**)&igLogButtons, "igLogButtons");
    lib.bindSymbol(cast(void**)&igDockBuilderRemoveNode, "igDockBuilderRemoveNode");
    lib.bindSymbol(cast(void**)&ImFont_ClearOutputData, "ImFont_ClearOutputData");
    lib.bindSymbol(cast(void**)&ImFont_ImFont, "ImFont_ImFont");
    lib.bindSymbol(cast(void**)&igEndTabItem, "igEndTabItem");
    lib.bindSymbol(cast(void**)&igVSliderFloat, "igVSliderFloat");
    lib.bindSymbol(cast(void**)&ImGuiIO_ClearInputKeys, "ImGuiIO_ClearInputKeys");
    lib.bindSymbol(cast(void**)&igRenderArrowPointingAt, "igRenderArrowPointingAt");
    lib.bindSymbol(cast(void**)&igEndGroup, "igEndGroup");
    lib.bindSymbol(cast(void**)&igPlotLines_FloatPtr, "igPlotLines_FloatPtr");
    lib.bindSymbol(cast(void**)&igPlotLines_FnFloatPtr, "igPlotLines_FnFloatPtr");
    lib.bindSymbol(cast(void**)&igGetColumnNormFromOffset, "igGetColumnNormFromOffset");
    lib.bindSymbol(cast(void**)&igBringWindowToDisplayBehind, "igBringWindowToDisplayBehind");
    lib.bindSymbol(cast(void**)&igSetItemAllowOverlap, "igSetItemAllowOverlap");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsCentralNode, "ImGuiDockNode_IsCentralNode");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetVoidPtrRef, "ImGuiStorage_GetVoidPtrRef");
    lib.bindSymbol(cast(void**)&igCheckboxFlags_IntPtr, "igCheckboxFlags_IntPtr");
    lib.bindSymbol(cast(void**)&igCheckboxFlags_UintPtr, "igCheckboxFlags_UintPtr");
    lib.bindSymbol(cast(void**)&igCheckboxFlags_S64Ptr, "igCheckboxFlags_S64Ptr");
    lib.bindSymbol(cast(void**)&igCheckboxFlags_U64Ptr, "igCheckboxFlags_U64Ptr");
    lib.bindSymbol(cast(void**)&ImRect_destroy, "ImRect_destroy");
    lib.bindSymbol(cast(void**)&igTreeNodeBehavior, "igTreeNodeBehavior");
    lib.bindSymbol(cast(void**)&igDebugTextEncoding, "igDebugTextEncoding");
    lib.bindSymbol(cast(void**)&igScrollToRect, "igScrollToRect");
    lib.bindSymbol(cast(void**)&igImTriangleBarycentricCoords, "igImTriangleBarycentricCoords");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_AddRanges, "ImFontGlyphRangesBuilder_AddRanges");
    lib.bindSymbol(cast(void**)&igTableSetBgColor, "igTableSetBgColor");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesVietnamese, "ImFontAtlas_GetGlyphRangesVietnamese");
    lib.bindSymbol(cast(void**)&ImGuiContextHook_ImGuiContextHook, "ImGuiContextHook_ImGuiContextHook");
    lib.bindSymbol(cast(void**)&igGetVersion, "igGetVersion");
    lib.bindSymbol(cast(void**)&ImDrawList_ImDrawList, "ImDrawList_ImDrawList");
    lib.bindSymbol(cast(void**)&igRenderTextEllipsis, "igRenderTextEllipsis");
    lib.bindSymbol(cast(void**)&ImGuiListClipper_destroy, "ImGuiListClipper_destroy");
    lib.bindSymbol(cast(void**)&igTableUpdateBorders, "igTableUpdateBorders");
    lib.bindSymbol(cast(void**)&ImGuiTableSortSpecs_destroy, "ImGuiTableSortSpecs_destroy");
    lib.bindSymbol(cast(void**)&igPushOverrideID, "igPushOverrideID");
    lib.bindSymbol(cast(void**)&igImMul, "igImMul");
    lib.bindSymbol(cast(void**)&igSetScrollY_Float, "igSetScrollY_Float");
    lib.bindSymbol(cast(void**)&igSetScrollY_WindowPtr, "igSetScrollY_WindowPtr");
    lib.bindSymbol(cast(void**)&ImFont_CalcWordWrapPositionA, "ImFont_CalcWordWrapPositionA");
    lib.bindSymbol(cast(void**)&igSmallButton, "igSmallButton");
    lib.bindSymbol(cast(void**)&ImGuiWindow_destroy, "ImGuiWindow_destroy");
    lib.bindSymbol(cast(void**)&ImGuiTableColumn_ImGuiTableColumn, "ImGuiTableColumn_ImGuiTableColumn");
    lib.bindSymbol(cast(void**)&ImGuiComboPreviewData_destroy, "ImGuiComboPreviewData_destroy");
    lib.bindSymbol(cast(void**)&igTableGetColumnResizeID, "igTableGetColumnResizeID");
    lib.bindSymbol(cast(void**)&igCombo_Str_arr, "igCombo_Str_arr");
    lib.bindSymbol(cast(void**)&igCombo_Str, "igCombo_Str");
    lib.bindSymbol(cast(void**)&igCombo_FnBoolPtr, "igCombo_FnBoolPtr");
    lib.bindSymbol(cast(void**)&igIsWindowChildOf, "igIsWindowChildOf");
    lib.bindSymbol(cast(void**)&ImGuiWindow_CalcFontSize, "ImGuiWindow_CalcFontSize");
    lib.bindSymbol(cast(void**)&igSetCurrentFont, "igSetCurrentFont");
    lib.bindSymbol(cast(void**)&igTableSetColumnWidth, "igTableSetColumnWidth");
    lib.bindSymbol(cast(void**)&ImDrawList_AddLine, "ImDrawList_AddLine");
    lib.bindSymbol(cast(void**)&ImDrawList_AddCircle, "ImDrawList_AddCircle");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_SelectAll, "ImGuiInputTextState_SelectAll");
    lib.bindSymbol(cast(void**)&igImParseFormatTrimDecorations, "igImParseFormatTrimDecorations");
    lib.bindSymbol(cast(void**)&ImGuiMetricsConfig_ImGuiMetricsConfig, "ImGuiMetricsConfig_ImGuiMetricsConfig");
    lib.bindSymbol(cast(void**)&ImGuiTabBar_ImGuiTabBar, "ImGuiTabBar_ImGuiTabBar");
    lib.bindSymbol(cast(void**)&ImGuiViewport_GetCenter, "ImGuiViewport_GetCenter");
    lib.bindSymbol(cast(void**)&igDebugDrawItemRect, "igDebugDrawItemRect");
    lib.bindSymbol(cast(void**)&igDockBuilderSetNodeSize, "igDockBuilderSetNodeSize");
    lib.bindSymbol(cast(void**)&igTreeNodeBehaviorIsOpen, "igTreeNodeBehaviorIsOpen");
    lib.bindSymbol(cast(void**)&igImTextCountUtf8BytesFromChar, "igImTextCountUtf8BytesFromChar");
    lib.bindSymbol(cast(void**)&igSetMouseCursor, "igSetMouseCursor");
    lib.bindSymbol(cast(void**)&igBeginColumns, "igBeginColumns");
    lib.bindSymbol(cast(void**)&igGetIO, "igGetIO");
    lib.bindSymbol(cast(void**)&igDragBehavior, "igDragBehavior");
    lib.bindSymbol(cast(void**)&igImModPositive, "igImModPositive");
    lib.bindSymbol(cast(void**)&ImFontAtlasCustomRect_destroy, "ImFontAtlasCustomRect_destroy");
    lib.bindSymbol(cast(void**)&ImGuiPayload_destroy, "ImGuiPayload_destroy");
    lib.bindSymbol(cast(void**)&igEndMenu, "igEndMenu");
    lib.bindSymbol(cast(void**)&igImSaturate, "igImSaturate");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimRect, "ImDrawList_PrimRect");
    lib.bindSymbol(cast(void**)&igImLinearSweep, "igImLinearSweep");
    lib.bindSymbol(cast(void**)&igUpdateMouseMovingWindowNewFrame, "igUpdateMouseMovingWindowNewFrame");
    lib.bindSymbol(cast(void**)&ImDrawList_AddRectFilled, "ImDrawList_AddRectFilled");
    lib.bindSymbol(cast(void**)&ImGuiPopupData_destroy, "ImGuiPopupData_destroy");
    lib.bindSymbol(cast(void**)&igFindSettingsHandler, "igFindSettingsHandler");
    lib.bindSymbol(cast(void**)&igDragInt2, "igDragInt2");
    lib.bindSymbol(cast(void**)&igBeginDocked, "igBeginDocked");
    lib.bindSymbol(cast(void**)&igSetColorEditOptions, "igSetColorEditOptions");
    lib.bindSymbol(cast(void**)&igIsAnyMouseDown, "igIsAnyMouseDown");
    lib.bindSymbol(cast(void**)&igRemoveSettingsHandler, "igRemoveSettingsHandler");
    lib.bindSymbol(cast(void**)&ImGuiDockContext_ImGuiDockContext, "ImGuiDockContext_ImGuiDockContext");
    lib.bindSymbol(cast(void**)&ImGuiTextFilter_Build, "ImGuiTextFilter_Build");
    lib.bindSymbol(cast(void**)&igTabItemCalcSize, "igTabItemCalcSize");
    lib.bindSymbol(cast(void**)&igSetNextWindowCollapsed, "igSetNextWindowCollapsed");
    lib.bindSymbol(cast(void**)&igSetLastItemData, "igSetLastItemData");
    lib.bindSymbol(cast(void**)&igLogToBuffer, "igLogToBuffer");
    lib.bindSymbol(cast(void**)&ImGuiTableTempData_destroy, "ImGuiTableTempData_destroy");
    lib.bindSymbol(cast(void**)&igImFileLoadToMemory, "igImFileLoadToMemory");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesCyrillic, "ImFontAtlas_GetGlyphRangesCyrillic");
    lib.bindSymbol(cast(void**)&ImGuiStyle_destroy, "ImGuiStyle_destroy");
    lib.bindSymbol(cast(void**)&ImDrawList_destroy, "ImDrawList_destroy");
    lib.bindSymbol(cast(void**)&ImVec4_destroy, "ImVec4_destroy");
    lib.bindSymbol(cast(void**)&igRenderCheckMark, "igRenderCheckMark");
    lib.bindSymbol(cast(void**)&igTreeNodeEx_Str, "igTreeNodeEx_Str");
    lib.bindSymbol(cast(void**)&igTreeNodeEx_StrStr, "igTreeNodeEx_StrStr");
    lib.bindSymbol(cast(void**)&igTreeNodeEx_Ptr, "igTreeNodeEx_Ptr");
    lib.bindSymbol(cast(void**)&ImBitVector_SetBit, "ImBitVector_SetBit");
    lib.bindSymbol(cast(void**)&igSetColumnWidth, "igSetColumnWidth");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_destroy, "ImGuiDockNode_destroy");
    lib.bindSymbol(cast(void**)&igIsItemClicked, "igIsItemClicked");
    lib.bindSymbol(cast(void**)&igTableOpenContextMenu, "igTableOpenContextMenu");
    lib.bindSymbol(cast(void**)&ImDrawList_AddCallback, "ImDrawList_AddCallback");
    lib.bindSymbol(cast(void**)&igGetMousePos, "igGetMousePos");
    lib.bindSymbol(cast(void**)&igDataTypeCompare, "igDataTypeCompare");
    lib.bindSymbol(cast(void**)&igDockContextQueueUndockNode, "igDockContextQueueUndockNode");
    lib.bindSymbol(cast(void**)&igImageButtonEx, "igImageButtonEx");
    lib.bindSymbol(cast(void**)&ImDrawListSharedData_SetCircleTessellationMaxError, "ImDrawListSharedData_SetCircleTessellationMaxError");
    lib.bindSymbol(cast(void**)&igBullet, "igBullet");
    lib.bindSymbol(cast(void**)&igRenderArrowDockMenu, "igRenderArrowDockMenu");
    lib.bindSymbol(cast(void**)&igTableSaveSettings, "igTableSaveSettings");
    lib.bindSymbol(cast(void**)&igTableGetBoundSettings, "igTableGetBoundSettings");
    lib.bindSymbol(cast(void**)&igGetHoveredID, "igGetHoveredID");
    lib.bindSymbol(cast(void**)&igGetWindowContentRegionMin, "igGetWindowContentRegionMin");
    lib.bindSymbol(cast(void**)&igTableHeadersRow, "igTableHeadersRow");
    lib.bindSymbol(cast(void**)&ImDrawList_AddNgonFilled, "ImDrawList_AddNgonFilled");
    lib.bindSymbol(cast(void**)&igDragScalar, "igDragScalar");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_ImGuiDockNode, "ImGuiDockNode_ImGuiDockNode");
    lib.bindSymbol(cast(void**)&igSetCursorPos, "igSetCursorPos");
    lib.bindSymbol(cast(void**)&igGcCompactTransientMiscBuffers, "igGcCompactTransientMiscBuffers");
    lib.bindSymbol(cast(void**)&igDebugHookIdInfo, "igDebugHookIdInfo");
    lib.bindSymbol(cast(void**)&igEndColumns, "igEndColumns");
    lib.bindSymbol(cast(void**)&igSetTooltip, "igSetTooltip");
    lib.bindSymbol(cast(void**)&igTableGetColumnName_Int, "igTableGetColumnName_Int");
    lib.bindSymbol(cast(void**)&igTableGetColumnName_TablePtr, "igTableGetColumnName_TablePtr");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_destroy, "ImGuiViewportP_destroy");
    lib.bindSymbol(cast(void**)&igBeginTabBarEx, "igBeginTabBarEx");
    lib.bindSymbol(cast(void**)&igShadeVertsLinearColorGradientKeepAlpha, "igShadeVertsLinearColorGradientKeepAlpha");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_HasSelection, "ImGuiInputTextState_HasSelection");
    lib.bindSymbol(cast(void**)&igDockNodeGetRootNode, "igDockNodeGetRootNode");
    lib.bindSymbol(cast(void**)&igWindowRectRelToAbs, "igWindowRectRelToAbs");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsSplitNode, "ImGuiDockNode_IsSplitNode");
    lib.bindSymbol(cast(void**)&igCalcItemWidth, "igCalcItemWidth");
    lib.bindSymbol(cast(void**)&igDockContextRebuildNodes, "igDockContextRebuildNodes");
    lib.bindSymbol(cast(void**)&ImGuiListClipperRange_FromIndices, "ImGuiListClipperRange_FromIndices");
    lib.bindSymbol(cast(void**)&igPushItemWidth, "igPushItemWidth");
    lib.bindSymbol(cast(void**)&igScrollbarEx, "igScrollbarEx");
    lib.bindSymbol(cast(void**)&ImDrawList_ChannelsMerge, "ImDrawList_ChannelsMerge");
    lib.bindSymbol(cast(void**)&igSetAllocatorFunctions, "igSetAllocatorFunctions");
    lib.bindSymbol(cast(void**)&ImFont_FindGlyph, "ImFont_FindGlyph");
    lib.bindSymbol(cast(void**)&igErrorCheckEndWindowRecover, "igErrorCheckEndWindowRecover");
    lib.bindSymbol(cast(void**)&igDockNodeGetDepth, "igDockNodeGetDepth");
    lib.bindSymbol(cast(void**)&igDebugStartItemPicker, "igDebugStartItemPicker");
    lib.bindSymbol(cast(void**)&ImGuiNextWindowData_destroy, "ImGuiNextWindowData_destroy");
    lib.bindSymbol(cast(void**)&ImGuiPayload_IsDelivery, "ImGuiPayload_IsDelivery");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesJapanese, "ImFontAtlas_GetGlyphRangesJapanese");
    lib.bindSymbol(cast(void**)&ImRect_Overlaps, "ImRect_Overlaps");
    lib.bindSymbol(cast(void**)&igCaptureMouseFromApp, "igCaptureMouseFromApp");
    lib.bindSymbol(cast(void**)&igAddContextHook, "igAddContextHook");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_GetCursorPos, "ImGuiInputTextState_GetCursorPos");
    lib.bindSymbol(cast(void**)&igImHashData, "igImHashData");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_GetBuildWorkRect, "ImGuiViewportP_GetBuildWorkRect");
    lib.bindSymbol(cast(void**)&ImGuiInputTextCallbackData_InsertChars, "ImGuiInputTextCallbackData_InsertChars");
    lib.bindSymbol(cast(void**)&igDragFloat2, "igDragFloat2");
    lib.bindSymbol(cast(void**)&igTreePushOverrideID, "igTreePushOverrideID");
    lib.bindSymbol(cast(void**)&igUpdateHoveredWindowAndCaptureFlags, "igUpdateHoveredWindowAndCaptureFlags");
    lib.bindSymbol(cast(void**)&ImFont_destroy, "ImFont_destroy");
    lib.bindSymbol(cast(void**)&igEndMenuBar, "igEndMenuBar");
    lib.bindSymbol(cast(void**)&igGetWindowSize, "igGetWindowSize");
    lib.bindSymbol(cast(void**)&igInputInt4, "igInputInt4");
    lib.bindSymbol(cast(void**)&igImSign_Float, "igImSign_Float");
    lib.bindSymbol(cast(void**)&igImSign_double, "igImSign_double");
    lib.bindSymbol(cast(void**)&ImDrawList_AddBezierQuadratic, "ImDrawList_AddBezierQuadratic");
    lib.bindSymbol(cast(void**)&igGetMouseCursor, "igGetMouseCursor");
    lib.bindSymbol(cast(void**)&igIsMouseDoubleClicked, "igIsMouseDoubleClicked");
    lib.bindSymbol(cast(void**)&igLabelText, "igLabelText");
    lib.bindSymbol(cast(void**)&ImDrawList_PathClear, "ImDrawList_PathClear");
    lib.bindSymbol(cast(void**)&ImDrawCmd_destroy, "ImDrawCmd_destroy");
    lib.bindSymbol(cast(void**)&igGetStateStorage, "igGetStateStorage");
    lib.bindSymbol(cast(void**)&igInputInt2, "igInputInt2");
    lib.bindSymbol(cast(void**)&igImFileRead, "igImFileRead");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildRender32bppRectFromString, "igImFontAtlasBuildRender32bppRectFromString");
    lib.bindSymbol(cast(void**)&ImGuiOldColumns_destroy, "ImGuiOldColumns_destroy");
    lib.bindSymbol(cast(void**)&igSetNextWindowScroll, "igSetNextWindowScroll");
    lib.bindSymbol(cast(void**)&igGetFrameHeight, "igGetFrameHeight");
    lib.bindSymbol(cast(void**)&igImFileWrite, "igImFileWrite");
    lib.bindSymbol(cast(void**)&igInputText, "igInputText");
    lib.bindSymbol(cast(void**)&igTreeNodeExV_Str, "igTreeNodeExV_Str");
    lib.bindSymbol(cast(void**)&igTreeNodeExV_Ptr, "igTreeNodeExV_Ptr");
    lib.bindSymbol(cast(void**)&igIsAnyItemFocused, "igIsAnyItemFocused");
    lib.bindSymbol(cast(void**)&ImDrawDataBuilder_Clear, "ImDrawDataBuilder_Clear");
    lib.bindSymbol(cast(void**)&ImVec2ih_ImVec2ih_Nil, "ImVec2ih_ImVec2ih_Nil");
    lib.bindSymbol(cast(void**)&ImVec2ih_ImVec2ih_short, "ImVec2ih_ImVec2ih_short");
    lib.bindSymbol(cast(void**)&ImVec2ih_ImVec2ih_Vec2, "ImVec2ih_ImVec2ih_Vec2");
    lib.bindSymbol(cast(void**)&igDockContextQueueDock, "igDockContextQueueDock");
    lib.bindSymbol(cast(void**)&igTableSetColumnSortDirection, "igTableSetColumnSortDirection");
    lib.bindSymbol(cast(void**)&ImVec1_ImVec1_Nil, "ImVec1_ImVec1_Nil");
    lib.bindSymbol(cast(void**)&ImVec1_ImVec1_Float, "ImVec1_ImVec1_Float");
    lib.bindSymbol(cast(void**)&igCalcItemSize, "igCalcItemSize");
    lib.bindSymbol(cast(void**)&ImFontAtlasCustomRect_IsPacked, "ImFontAtlasCustomRect_IsPacked");
    lib.bindSymbol(cast(void**)&igPopStyleColor, "igPopStyleColor");
    lib.bindSymbol(cast(void**)&igColorEdit4, "igColorEdit4");
    lib.bindSymbol(cast(void**)&igPlotEx, "igPlotEx");
    lib.bindSymbol(cast(void**)&igGetCursorStartPos, "igGetCursorStartPos");
    lib.bindSymbol(cast(void**)&igShowFontAtlas, "igShowFontAtlas");
    lib.bindSymbol(cast(void**)&igDockSpaceOverViewport, "igDockSpaceOverViewport");
    lib.bindSymbol(cast(void**)&ImGuiInputTextCallbackData_destroy, "ImGuiInputTextCallbackData_destroy");
    lib.bindSymbol(cast(void**)&ImFontAtlas_IsBuilt, "ImFontAtlas_IsBuilt");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_begin, "ImGuiTextBuffer_begin");
    lib.bindSymbol(cast(void**)&ImGuiTable_destroy, "ImGuiTable_destroy");
    lib.bindSymbol(cast(void**)&ImVec4_ImVec4_Nil, "ImVec4_ImVec4_Nil");
    lib.bindSymbol(cast(void**)&ImVec4_ImVec4_Float, "ImVec4_ImVec4_Float");
    lib.bindSymbol(cast(void**)&ImFont_BuildLookupTable, "ImFont_BuildLookupTable");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_appendfv, "ImGuiTextBuffer_appendfv");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsEmpty, "ImGuiDockNode_IsEmpty");
    lib.bindSymbol(cast(void**)&igClearIniSettings, "igClearIniSettings");
    lib.bindSymbol(cast(void**)&ImDrawList_PathLineToMergeDuplicate, "ImDrawList_PathLineToMergeDuplicate");
    lib.bindSymbol(cast(void**)&ImGuiIO_ImGuiIO, "ImGuiIO_ImGuiIO");
    lib.bindSymbol(cast(void**)&igDragInt4, "igDragInt4");
    lib.bindSymbol(cast(void**)&igBeginDragDropTarget, "igBeginDragDropTarget");
    lib.bindSymbol(cast(void**)&igImTextCountCharsFromUtf8, "igImTextCountCharsFromUtf8");
    lib.bindSymbol(cast(void**)&igTablePopBackgroundChannel, "igTablePopBackgroundChannel");
    lib.bindSymbol(cast(void**)&igSetNextWindowClass, "igSetNextWindowClass");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_clear, "ImGuiTextBuffer_clear");
    lib.bindSymbol(cast(void**)&igImStricmp, "igImStricmp");
    lib.bindSymbol(cast(void**)&igMarkItemEdited, "igMarkItemEdited");
    lib.bindSymbol(cast(void**)&igIsWindowFocused, "igIsWindowFocused");
    lib.bindSymbol(cast(void**)&igTableSettingsCreate, "igTableSettingsCreate");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddInputCharactersUTF8, "ImGuiIO_AddInputCharactersUTF8");
    lib.bindSymbol(cast(void**)&ImGuiTableSettings_destroy, "ImGuiTableSettings_destroy");
    lib.bindSymbol(cast(void**)&igIsWindowAbove, "igIsWindowAbove");
    lib.bindSymbol(cast(void**)&ImDrawList__PathArcToN, "ImDrawList__PathArcToN");
    lib.bindSymbol(cast(void**)&igColorTooltip, "igColorTooltip");
    lib.bindSymbol(cast(void**)&igSetCurrentContext, "igSetCurrentContext");
    lib.bindSymbol(cast(void**)&igImTriangleClosestPoint, "igImTriangleClosestPoint");
    lib.bindSymbol(cast(void**)&igSliderInt4, "igSliderInt4");
    lib.bindSymbol(cast(void**)&igGetItemRectMin, "igGetItemRectMin");
    lib.bindSymbol(cast(void**)&igTableUpdateColumnsWeightFromWidth, "igTableUpdateColumnsWeightFromWidth");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimReserve, "ImDrawList_PrimReserve");
    lib.bindSymbol(cast(void**)&ImGuiMenuColumns_ImGuiMenuColumns, "ImGuiMenuColumns_ImGuiMenuColumns");
    lib.bindSymbol(cast(void**)&igDockBuilderGetCentralNode, "igDockBuilderGetCentralNode");
    lib.bindSymbol(cast(void**)&ImDrawList_AddRectFilledMultiColor, "ImDrawList_AddRectFilledMultiColor");
    lib.bindSymbol(cast(void**)&ImGuiDockNodeSettings_destroy, "ImGuiDockNodeSettings_destroy");
    lib.bindSymbol(cast(void**)&igGetWindowViewport, "igGetWindowViewport");
    lib.bindSymbol(cast(void**)&igSetStateStorage, "igSetStateStorage");
    lib.bindSymbol(cast(void**)&ImGuiStorage_SetAllInt, "ImGuiStorage_SetAllInt");
    lib.bindSymbol(cast(void**)&ImGuiListClipper_Step, "ImGuiListClipper_Step");
    lib.bindSymbol(cast(void**)&ImGuiOnceUponAFrame_destroy, "ImGuiOnceUponAFrame_destroy");
    lib.bindSymbol(cast(void**)&ImGuiInputTextCallbackData_DeleteChars, "ImGuiInputTextCallbackData_DeleteChars");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildSetupFont, "igImFontAtlasBuildSetupFont");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_empty, "ImGuiTextBuffer_empty");
    lib.bindSymbol(cast(void**)&igShowDemoWindow, "igShowDemoWindow");
    lib.bindSymbol(cast(void**)&igImPow_Float, "igImPow_Float");
    lib.bindSymbol(cast(void**)&igImPow_double, "igImPow_double");
    lib.bindSymbol(cast(void**)&ImGuiTextRange_destroy, "ImGuiTextRange_destroy");
    lib.bindSymbol(cast(void**)&ImGuiStorage_SetVoidPtr, "ImGuiStorage_SetVoidPtr");
    lib.bindSymbol(cast(void**)&igDockContextEndFrame, "igDockContextEndFrame");
    lib.bindSymbol(cast(void**)&igFindWindowDisplayIndex, "igFindWindowDisplayIndex");
    lib.bindSymbol(cast(void**)&igGetFocusScope, "igGetFocusScope");
    lib.bindSymbol(cast(void**)&igCloseButton, "igCloseButton");
    lib.bindSymbol(cast(void**)&igImInvLength, "igImInvLength");
    lib.bindSymbol(cast(void**)&ImDrawList_PushTextureID, "ImDrawList_PushTextureID");
    lib.bindSymbol(cast(void**)&ImDrawList_PathLineTo, "ImDrawList_PathLineTo");
    lib.bindSymbol(cast(void**)&igSetWindowHitTestHole, "igSetWindowHitTestHole");
    lib.bindSymbol(cast(void**)&igSeparatorEx, "igSeparatorEx");
    lib.bindSymbol(cast(void**)&ImRect_Add_Vec2, "ImRect_Add_Vec2");
    lib.bindSymbol(cast(void**)&ImRect_Add_Rect, "ImRect_Add_Rect");
    lib.bindSymbol(cast(void**)&igShowMetricsWindow, "igShowMetricsWindow");
    lib.bindSymbol(cast(void**)&ImDrawList__PopUnusedDrawCmd, "ImDrawList__PopUnusedDrawCmd");
    lib.bindSymbol(cast(void**)&ImDrawList_AddImageRounded, "ImDrawList_AddImageRounded");
    lib.bindSymbol(cast(void**)&ImGuiStyleMod_destroy, "ImGuiStyleMod_destroy");
    lib.bindSymbol(cast(void**)&ImGuiMenuColumns_CalcNextTotalWidth, "ImGuiMenuColumns_CalcNextTotalWidth");
    lib.bindSymbol(cast(void**)&ImGuiStorage_BuildSortByKey, "ImGuiStorage_BuildSortByKey");
    lib.bindSymbol(cast(void**)&ImDrawList_PathRect, "ImDrawList_PathRect");
    lib.bindSymbol(cast(void**)&igInputTextEx, "igInputTextEx");
    lib.bindSymbol(cast(void**)&igAddSettingsHandler, "igAddSettingsHandler");
    lib.bindSymbol(cast(void**)&igColorEdit3, "igColorEdit3");
    lib.bindSymbol(cast(void**)&ImColor_destroy, "ImColor_destroy");
    lib.bindSymbol(cast(void**)&igIsItemToggledSelection, "igIsItemToggledSelection");
    lib.bindSymbol(cast(void**)&igTabItemEx, "igTabItemEx");
    lib.bindSymbol(cast(void**)&igIsKeyPressedMap, "igIsKeyPressedMap");
    lib.bindSymbol(cast(void**)&igIsNamedKey, "igIsNamedKey");
    lib.bindSymbol(cast(void**)&igTableSetupDrawChannels, "igTableSetupDrawChannels");
    lib.bindSymbol(cast(void**)&igLogFinish, "igLogFinish");
    lib.bindSymbol(cast(void**)&igGetItemRectSize, "igGetItemRectSize");
    lib.bindSymbol(cast(void**)&igGetWindowResizeCornerID, "igGetWindowResizeCornerID");
    lib.bindSymbol(cast(void**)&ImGuiListClipperData_destroy, "ImGuiListClipperData_destroy");
    lib.bindSymbol(cast(void**)&igImParseFormatFindStart, "igImParseFormatFindStart");
    lib.bindSymbol(cast(void**)&ImGuiDockRequest_ImGuiDockRequest, "ImGuiDockRequest_ImGuiDockRequest");
    lib.bindSymbol(cast(void**)&ImDrawData_ImDrawData, "ImDrawData_ImDrawData");
    lib.bindSymbol(cast(void**)&igDockNodeEndAmendTabBar, "igDockNodeEndAmendTabBar");
    lib.bindSymbol(cast(void**)&igDragScalarN, "igDragScalarN");
    lib.bindSymbol(cast(void**)&igImDot, "igImDot");
    lib.bindSymbol(cast(void**)&igMarkIniSettingsDirty_Nil, "igMarkIniSettingsDirty_Nil");
    lib.bindSymbol(cast(void**)&igMarkIniSettingsDirty_WindowPtr, "igMarkIniSettingsDirty_WindowPtr");
    lib.bindSymbol(cast(void**)&igTableGetColumnCount, "igTableGetColumnCount");
    lib.bindSymbol(cast(void**)&igGetWindowWidth, "igGetWindowWidth");
    lib.bindSymbol(cast(void**)&igBulletTextV, "igBulletTextV");
    lib.bindSymbol(cast(void**)&igDockBuilderCopyNode, "igDockBuilderCopyNode");
    lib.bindSymbol(cast(void**)&ImDrawListSplitter_SetCurrentChannel, "ImDrawListSplitter_SetCurrentChannel");
    lib.bindSymbol(cast(void**)&ImGuiStorage_SetBool, "ImGuiStorage_SetBool");
    lib.bindSymbol(cast(void**)&igAlignTextToFramePadding, "igAlignTextToFramePadding");
    lib.bindSymbol(cast(void**)&igIsWindowHovered, "igIsWindowHovered");
    lib.bindSymbol(cast(void**)&igDockBuilderCopyDockSpace, "igDockBuilderCopyDockSpace");
    lib.bindSymbol(cast(void**)&ImGuiTableTempData_ImGuiTableTempData, "ImGuiTableTempData_ImGuiTableTempData");
    lib.bindSymbol(cast(void**)&ImRect_GetCenter, "ImRect_GetCenter");
    lib.bindSymbol(cast(void**)&igDockNodeIsInHierarchyOf, "igDockNodeIsInHierarchyOf");
    lib.bindSymbol(cast(void**)&ImDrawList_PathArcTo, "ImDrawList_PathArcTo");
    lib.bindSymbol(cast(void**)&igIsAnyItemActive, "igIsAnyItemActive");
    lib.bindSymbol(cast(void**)&igPushTextWrapPos, "igPushTextWrapPos");
    lib.bindSymbol(cast(void**)&igGetTreeNodeToLabelSpacing, "igGetTreeNodeToLabelSpacing");
    lib.bindSymbol(cast(void**)&igSameLine, "igSameLine");
    lib.bindSymbol(cast(void**)&igSetWindowViewport, "igSetWindowViewport");
    lib.bindSymbol(cast(void**)&igDebugNodeDockNode, "igDebugNodeDockNode");
    lib.bindSymbol(cast(void**)&igDummy, "igDummy");
    lib.bindSymbol(cast(void**)&igStyleColorsDark, "igStyleColorsDark");
    lib.bindSymbol(cast(void**)&igGetItemID, "igGetItemID");
    lib.bindSymbol(cast(void**)&igImageButton, "igImageButton");
    lib.bindSymbol(cast(void**)&igGetWindowContentRegionMax, "igGetWindowContentRegionMax");
    lib.bindSymbol(cast(void**)&igTabBarQueueReorder, "igTabBarQueueReorder");
    lib.bindSymbol(cast(void**)&igGetKeyPressedAmount, "igGetKeyPressedAmount");
    lib.bindSymbol(cast(void**)&igRenderTextClipped, "igRenderTextClipped");
    lib.bindSymbol(cast(void**)&igImIsPowerOfTwo_Int, "igImIsPowerOfTwo_Int");
    lib.bindSymbol(cast(void**)&igImIsPowerOfTwo_U64, "igImIsPowerOfTwo_U64");
    lib.bindSymbol(cast(void**)&igSetNextWindowSizeConstraints, "igSetNextWindowSizeConstraints");
    lib.bindSymbol(cast(void**)&igTableGcCompactTransientBuffers_TablePtr, "igTableGcCompactTransientBuffers_TablePtr");
    lib.bindSymbol(cast(void**)&igTableGcCompactTransientBuffers_TableTempDataPtr, "igTableGcCompactTransientBuffers_TableTempDataPtr");
    lib.bindSymbol(cast(void**)&ImFont_FindGlyphNoFallback, "ImFont_FindGlyphNoFallback");
    lib.bindSymbol(cast(void**)&igShowStyleSelector, "igShowStyleSelector");
    lib.bindSymbol(cast(void**)&igNavMoveRequestForward, "igNavMoveRequestForward");
    lib.bindSymbol(cast(void**)&ImGuiListClipperData_Reset, "ImGuiListClipperData_Reset");
    lib.bindSymbol(cast(void**)&igNavInitWindow, "igNavInitWindow");
    lib.bindSymbol(cast(void**)&igImFileOpen, "igImFileOpen");
    lib.bindSymbol(cast(void**)&igEndDragDropTarget, "igEndDragDropTarget");
    lib.bindSymbol(cast(void**)&ImGuiWindowSettings_ImGuiWindowSettings, "ImGuiWindowSettings_ImGuiWindowSettings");
    lib.bindSymbol(cast(void**)&ImDrawListSharedData_destroy, "ImDrawListSharedData_destroy");
    lib.bindSymbol(cast(void**)&ImFontAtlas_Build, "ImFontAtlas_Build");
    lib.bindSymbol(cast(void**)&ImGuiDockPreviewData_destroy, "ImGuiDockPreviewData_destroy");
    lib.bindSymbol(cast(void**)&igSetScrollFromPosX_Float, "igSetScrollFromPosX_Float");
    lib.bindSymbol(cast(void**)&igSetScrollFromPosX_WindowPtr, "igSetScrollFromPosX_WindowPtr");
    lib.bindSymbol(cast(void**)&igIsKeyPressed, "igIsKeyPressed");
    lib.bindSymbol(cast(void**)&igEndTooltip, "igEndTooltip");
    lib.bindSymbol(cast(void**)&igFindWindowSettings, "igFindWindowSettings");
    lib.bindSymbol(cast(void**)&igDebugRenderViewportThumbnail, "igDebugRenderViewportThumbnail");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_UpdateMergedFlags, "ImGuiDockNode_UpdateMergedFlags");
    lib.bindSymbol(cast(void**)&igKeepAliveID, "igKeepAliveID");
    lib.bindSymbol(cast(void**)&igGetColumnOffsetFromNorm, "igGetColumnOffsetFromNorm");
    lib.bindSymbol(cast(void**)&ImFont_IsLoaded, "ImFont_IsLoaded");
    lib.bindSymbol(cast(void**)&igDebugNodeDrawCmdShowMeshAndBoundingBox, "igDebugNodeDrawCmdShowMeshAndBoundingBox");
    lib.bindSymbol(cast(void**)&igBeginDragDropSource, "igBeginDragDropSource");
    lib.bindSymbol(cast(void**)&ImBitVector_ClearBit, "ImBitVector_ClearBit");
    lib.bindSymbol(cast(void**)&ImDrawDataBuilder_GetDrawListCount, "ImDrawDataBuilder_GetDrawListCount");
    lib.bindSymbol(cast(void**)&ImGuiDockRequest_destroy, "ImGuiDockRequest_destroy");
    lib.bindSymbol(cast(void**)&igSetScrollFromPosY_Float, "igSetScrollFromPosY_Float");
    lib.bindSymbol(cast(void**)&igSetScrollFromPosY_WindowPtr, "igSetScrollFromPosY_WindowPtr");
    lib.bindSymbol(cast(void**)&igColorButton, "igColorButton");
    lib.bindSymbol(cast(void**)&igAcceptDragDropPayload, "igAcceptDragDropPayload");
    lib.bindSymbol(cast(void**)&igDockContextShutdown, "igDockContextShutdown");
    lib.bindSymbol(cast(void**)&ImDrawList_PopClipRect, "ImDrawList_PopClipRect");
    lib.bindSymbol(cast(void**)&igGetCursorPosX, "igGetCursorPosX");
    lib.bindSymbol(cast(void**)&igGetScrollMaxY, "igGetScrollMaxY");
    lib.bindSymbol(cast(void**)&ImGuiStoragePair_ImGuiStoragePair_Int, "ImGuiStoragePair_ImGuiStoragePair_Int");
    lib.bindSymbol(cast(void**)&ImGuiStoragePair_ImGuiStoragePair_Float, "ImGuiStoragePair_ImGuiStoragePair_Float");
    lib.bindSymbol(cast(void**)&ImGuiStoragePair_ImGuiStoragePair_Ptr, "ImGuiStoragePair_ImGuiStoragePair_Ptr");
    lib.bindSymbol(cast(void**)&igEndMainMenuBar, "igEndMainMenuBar");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_GetMainRect, "ImGuiViewportP_GetMainRect");
    lib.bindSymbol(cast(void**)&ImGuiPlatformMonitor_ImGuiPlatformMonitor, "ImGuiPlatformMonitor_ImGuiPlatformMonitor");
    lib.bindSymbol(cast(void**)&igIsItemActive, "igIsItemActive");
    lib.bindSymbol(cast(void**)&ImRect_GetArea, "ImRect_GetArea");
    lib.bindSymbol(cast(void**)&igShowAboutWindow, "igShowAboutWindow");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_GetSelectionStart, "ImGuiInputTextState_GetSelectionStart");
    lib.bindSymbol(cast(void**)&igPushFont, "igPushFont");
    lib.bindSymbol(cast(void**)&igImStrchrRange, "igImStrchrRange");
    lib.bindSymbol(cast(void**)&igSetNextItemWidth, "igSetNextItemWidth");
    lib.bindSymbol(cast(void**)&igGetContentRegionAvail, "igGetContentRegionAvail");
    lib.bindSymbol(cast(void**)&ImGuiPayload_ImGuiPayload, "ImGuiPayload_ImGuiPayload");
    lib.bindSymbol(cast(void**)&igCheckbox, "igCheckbox");
    lib.bindSymbol(cast(void**)&ImGuiListClipperData_ImGuiListClipperData, "ImGuiListClipperData_ImGuiListClipperData");
    lib.bindSymbol(cast(void**)&ImGuiTextRange_ImGuiTextRange_Nil, "ImGuiTextRange_ImGuiTextRange_Nil");
    lib.bindSymbol(cast(void**)&ImGuiTextRange_ImGuiTextRange_Str, "ImGuiTextRange_ImGuiTextRange_Str");
    lib.bindSymbol(cast(void**)&ImFontAtlas_destroy, "ImFontAtlas_destroy");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddMouseViewportEvent, "ImGuiIO_AddMouseViewportEvent");
    lib.bindSymbol(cast(void**)&ImGuiMenuColumns_Update, "ImGuiMenuColumns_Update");
    lib.bindSymbol(cast(void**)&igGcCompactTransientWindowBuffers, "igGcCompactTransientWindowBuffers");
    lib.bindSymbol(cast(void**)&igTableSortSpecsBuild, "igTableSortSpecsBuild");
    lib.bindSymbol(cast(void**)&igNavMoveRequestTryWrapping, "igNavMoveRequestTryWrapping");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_GetSelectionEnd, "ImGuiInputTextState_GetSelectionEnd");
    lib.bindSymbol(cast(void**)&igIsWindowDocked, "igIsWindowDocked");
    lib.bindSymbol(cast(void**)&ImVec2_destroy, "ImVec2_destroy");
    lib.bindSymbol(cast(void**)&igTableBeginRow, "igTableBeginRow");
    lib.bindSymbol(cast(void**)&igGetCurrentWindow, "igGetCurrentWindow");
    lib.bindSymbol(cast(void**)&igSetDragDropPayload, "igSetDragDropPayload");
    lib.bindSymbol(cast(void**)&igGetID_Str, "igGetID_Str");
    lib.bindSymbol(cast(void**)&igGetID_StrStr, "igGetID_StrStr");
    lib.bindSymbol(cast(void**)&igGetID_Ptr, "igGetID_Ptr");
    lib.bindSymbol(cast(void**)&ImFontAtlas_ImFontAtlas, "ImFontAtlas_ImFontAtlas");
    lib.bindSymbol(cast(void**)&igBeginGroup, "igBeginGroup");
    lib.bindSymbol(cast(void**)&igGetContentRegionMax, "igGetContentRegionMax");
    lib.bindSymbol(cast(void**)&igEndChildFrame, "igEndChildFrame");
    lib.bindSymbol(cast(void**)&igActivateItem, "igActivateItem");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildMultiplyCalcLookupTable, "igImFontAtlasBuildMultiplyCalcLookupTable");
    lib.bindSymbol(cast(void**)&ImDrawList_PushClipRectFullScreen, "ImDrawList_PushClipRectFullScreen");
    lib.bindSymbol(cast(void**)&ImRect_Contains_Vec2, "ImRect_Contains_Vec2");
    lib.bindSymbol(cast(void**)&ImRect_Contains_Rect, "ImRect_Contains_Rect");
    lib.bindSymbol(cast(void**)&igGetBackgroundDrawList_Nil, "igGetBackgroundDrawList_Nil");
    lib.bindSymbol(cast(void**)&igGetBackgroundDrawList_ViewportPtr, "igGetBackgroundDrawList_ViewportPtr");
    lib.bindSymbol(cast(void**)&igSetColumnOffset, "igSetColumnOffset");
    lib.bindSymbol(cast(void**)&igSetKeyboardFocusHere, "igSetKeyboardFocusHere");
    lib.bindSymbol(cast(void**)&igLoadIniSettingsFromMemory, "igLoadIniSettingsFromMemory");
    lib.bindSymbol(cast(void**)&igIndent, "igIndent");
    lib.bindSymbol(cast(void**)&igPopStyleVar, "igPopStyleVar");
    lib.bindSymbol(cast(void**)&igGetViewportPlatformMonitor, "igGetViewportPlatformMonitor");
    lib.bindSymbol(cast(void**)&igSetNextWindowSize, "igSetNextWindowSize");
    lib.bindSymbol(cast(void**)&igInputFloat3, "igInputFloat3");
    lib.bindSymbol(cast(void**)&igIsKeyDown, "igIsKeyDown");
    lib.bindSymbol(cast(void**)&igTableBeginApplyRequests, "igTableBeginApplyRequests");
    lib.bindSymbol(cast(void**)&igDockNodeBeginAmendTabBar, "igDockNodeBeginAmendTabBar");
    lib.bindSymbol(cast(void**)&igBeginMenuEx, "igBeginMenuEx");
    lib.bindSymbol(cast(void**)&igTextUnformatted, "igTextUnformatted");
    lib.bindSymbol(cast(void**)&igTextV, "igTextV");
    lib.bindSymbol(cast(void**)&igImLengthSqr_Vec2, "igImLengthSqr_Vec2");
    lib.bindSymbol(cast(void**)&igImLengthSqr_Vec4, "igImLengthSqr_Vec4");
    lib.bindSymbol(cast(void**)&ImGuiTextFilter_Draw, "ImGuiTextFilter_Draw");
    lib.bindSymbol(cast(void**)&igFocusWindow, "igFocusWindow");
    lib.bindSymbol(cast(void**)&igPushClipRect, "igPushClipRect");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_ImGuiViewportP, "ImGuiViewportP_ImGuiViewportP");
    lib.bindSymbol(cast(void**)&igBeginMainMenuBar, "igBeginMainMenuBar");
    lib.bindSymbol(cast(void**)&ImRect_GetBR, "ImRect_GetBR");
    lib.bindSymbol(cast(void**)&igCollapsingHeader_TreeNodeFlags, "igCollapsingHeader_TreeNodeFlags");
    lib.bindSymbol(cast(void**)&igCollapsingHeader_BoolPtr, "igCollapsingHeader_BoolPtr");
    lib.bindSymbol(cast(void**)&igGetCurrentWindowRead, "igGetCurrentWindowRead");
    lib.bindSymbol(cast(void**)&ImDrawList__PathArcToFastEx, "ImDrawList__PathArcToFastEx");
    lib.bindSymbol(cast(void**)&igSliderInt3, "igSliderInt3");
    lib.bindSymbol(cast(void**)&igTabBarAddTab, "igTabBarAddTab");
    lib.bindSymbol(cast(void**)&ImGuiTableSettings_ImGuiTableSettings, "ImGuiTableSettings_ImGuiTableSettings");
    lib.bindSymbol(cast(void**)&igPushStyleColor_U32, "igPushStyleColor_U32");
    lib.bindSymbol(cast(void**)&igPushStyleColor_Vec4, "igPushStyleColor_Vec4");
    lib.bindSymbol(cast(void**)&igImFormatString, "igImFormatString");
    lib.bindSymbol(cast(void**)&igTabItemButton, "igTabItemButton");
    lib.bindSymbol(cast(void**)&igIsMouseReleased, "igIsMouseReleased");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_CursorClamp, "ImGuiInputTextState_CursorClamp");
    lib.bindSymbol(cast(void**)&igRemoveContextHook, "igRemoveContextHook");
    lib.bindSymbol(cast(void**)&ImFontAtlasCustomRect_ImFontAtlasCustomRect, "ImFontAtlasCustomRect_ImFontAtlasCustomRect");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddInputCharacter, "ImGuiIO_AddInputCharacter");
    lib.bindSymbol(cast(void**)&igTabBarProcessReorder, "igTabBarProcessReorder");
    lib.bindSymbol(cast(void**)&igGetNavInputAmount, "igGetNavInputAmount");
    lib.bindSymbol(cast(void**)&igClearDragDrop, "igClearDragDrop");
    lib.bindSymbol(cast(void**)&igGetTextLineHeight, "igGetTextLineHeight");
    lib.bindSymbol(cast(void**)&ImDrawList_AddBezierCubic, "ImDrawList_AddBezierCubic");
    lib.bindSymbol(cast(void**)&igDockContextNewFrameUpdateDocking, "igDockContextNewFrameUpdateDocking");
    lib.bindSymbol(cast(void**)&igDataTypeApplyOp, "igDataTypeApplyOp");
    lib.bindSymbol(cast(void**)&ImDrawList_AddQuadFilled, "ImDrawList_AddQuadFilled");
    lib.bindSymbol(cast(void**)&igDockContextNewFrameUpdateUndocking, "igDockContextNewFrameUpdateUndocking");
    lib.bindSymbol(cast(void**)&ImGuiInputTextCallbackData_SelectAll, "ImGuiInputTextCallbackData_SelectAll");
    lib.bindSymbol(cast(void**)&ImGuiNextItemData_ImGuiNextItemData, "ImGuiNextItemData_ImGuiNextItemData");
    lib.bindSymbol(cast(void**)&igLogRenderedText, "igLogRenderedText");
    lib.bindSymbol(cast(void**)&igBeginMenu, "igBeginMenu");
    lib.bindSymbol(cast(void**)&igSetNextWindowBgAlpha, "igSetNextWindowBgAlpha");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetIntRef, "ImGuiStorage_GetIntRef");
    lib.bindSymbol(cast(void**)&igImTextCountUtf8BytesFromStr, "igImTextCountUtf8BytesFromStr");
    lib.bindSymbol(cast(void**)&igEndCombo, "igEndCombo");
    lib.bindSymbol(cast(void**)&igIsNavInputTest, "igIsNavInputTest");
    lib.bindSymbol(cast(void**)&ImGuiStackTool_ImGuiStackTool, "ImGuiStackTool_ImGuiStackTool");
    lib.bindSymbol(cast(void**)&ImDrawList_AddPolyline, "ImDrawList_AddPolyline");
    lib.bindSymbol(cast(void**)&igImage, "igImage");
    lib.bindSymbol(cast(void**)&igPopClipRect, "igPopClipRect");
    lib.bindSymbol(cast(void**)&ImDrawList_PushClipRect, "ImDrawList_PushClipRect");
    lib.bindSymbol(cast(void**)&igImBitArrayClearBit, "igImBitArrayClearBit");
    lib.bindSymbol(cast(void**)&igArrowButtonEx, "igArrowButtonEx");
    lib.bindSymbol(cast(void**)&igSelectable_Bool, "igSelectable_Bool");
    lib.bindSymbol(cast(void**)&igSelectable_BoolPtr, "igSelectable_BoolPtr");
    lib.bindSymbol(cast(void**)&igTableSetColumnWidthAutoSingle, "igTableSetColumnWidthAutoSingle");
    lib.bindSymbol(cast(void**)&igBeginTooltipEx, "igBeginTooltipEx");
    lib.bindSymbol(cast(void**)&igTreeNode_Str, "igTreeNode_Str");
    lib.bindSymbol(cast(void**)&igTreeNode_StrStr, "igTreeNode_StrStr");
    lib.bindSymbol(cast(void**)&igTreeNode_Ptr, "igTreeNode_Ptr");
    lib.bindSymbol(cast(void**)&igGetFocusID, "igGetFocusID");
    lib.bindSymbol(cast(void**)&ImGuiTableInstanceData_ImGuiTableInstanceData, "ImGuiTableInstanceData_ImGuiTableInstanceData");
    lib.bindSymbol(cast(void**)&igEndComboPreview, "igEndComboPreview");
    lib.bindSymbol(cast(void**)&ImDrawData_DeIndexAllBuffers, "ImDrawData_DeIndexAllBuffers");
    lib.bindSymbol(cast(void**)&ImDrawCmd_ImDrawCmd, "ImDrawCmd_ImDrawCmd");
    lib.bindSymbol(cast(void**)&ImDrawData_ScaleClipRects, "ImDrawData_ScaleClipRects");
    lib.bindSymbol(cast(void**)&igBeginViewportSideBar, "igBeginViewportSideBar");
    lib.bindSymbol(cast(void**)&igSetNextItemOpen, "igSetNextItemOpen");
    lib.bindSymbol(cast(void**)&igDataTypeFormatString, "igDataTypeFormatString");
    lib.bindSymbol(cast(void**)&igTabItemBackground, "igTabItemBackground");
    lib.bindSymbol(cast(void**)&ImDrawList_AddTriangle, "ImDrawList_AddTriangle");
    lib.bindSymbol(cast(void**)&igDockContextClearNodes, "igDockContextClearNodes");
    lib.bindSymbol(cast(void**)&ImGuiContextHook_destroy, "ImGuiContextHook_destroy");
    lib.bindSymbol(cast(void**)&igLogToFile, "igLogToFile");
    lib.bindSymbol(cast(void**)&igGetWindowResizeBorderID, "igGetWindowResizeBorderID");
    lib.bindSymbol(cast(void**)&ImGuiNextItemData_destroy, "ImGuiNextItemData_destroy");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_ClearRequestFlags, "ImGuiViewportP_ClearRequestFlags");
    lib.bindSymbol(cast(void**)&igTempInputIsActive, "igTempInputIsActive");
    lib.bindSymbol(cast(void**)&ImDrawCmd_GetTexID, "ImDrawCmd_GetTexID");
    lib.bindSymbol(cast(void**)&ImGuiIO_SetKeyEventNativeData, "ImGuiIO_SetKeyEventNativeData");
    lib.bindSymbol(cast(void**)&igDebugNodeWindowSettings, "igDebugNodeWindowSettings");
    lib.bindSymbol(cast(void**)&igSetNextWindowDockID, "igSetNextWindowDockID");
    lib.bindSymbol(cast(void**)&ImRect_ToVec4, "ImRect_ToVec4");
    lib.bindSymbol(cast(void**)&igTableGcCompactSettings, "igTableGcCompactSettings");
    lib.bindSymbol(cast(void**)&igPushMultiItemsWidths, "igPushMultiItemsWidths");
    lib.bindSymbol(cast(void**)&igCreateContext, "igCreateContext");
    lib.bindSymbol(cast(void**)&igTableNextRow, "igTableNextRow");
    lib.bindSymbol(cast(void**)&ImGuiStackSizes_CompareWithCurrentState, "ImGuiStackSizes_CompareWithCurrentState");
    lib.bindSymbol(cast(void**)&ImColor_ImColor_Nil, "ImColor_ImColor_Nil");
    lib.bindSymbol(cast(void**)&ImColor_ImColor_Float, "ImColor_ImColor_Float");
    lib.bindSymbol(cast(void**)&ImColor_ImColor_Vec4, "ImColor_ImColor_Vec4");
    lib.bindSymbol(cast(void**)&ImColor_ImColor_Int, "ImColor_ImColor_Int");
    lib.bindSymbol(cast(void**)&ImColor_ImColor_U32, "ImColor_ImColor_U32");
    lib.bindSymbol(cast(void**)&igTableGetMaxColumnWidth, "igTableGetMaxColumnWidth");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_CalcWorkRectPos, "ImGuiViewportP_CalcWorkRectPos");
    lib.bindSymbol(cast(void**)&ImGuiStackTool_destroy, "ImGuiStackTool_destroy");
    lib.bindSymbol(cast(void**)&igDockContextGenNodeID, "igDockContextGenNodeID");
    lib.bindSymbol(cast(void**)&ImDrawList__ClearFreeMemory, "ImDrawList__ClearFreeMemory");
    lib.bindSymbol(cast(void**)&igSetNavID, "igSetNavID");
    lib.bindSymbol(cast(void**)&igGetWindowDrawList, "igGetWindowDrawList");
    lib.bindSymbol(cast(void**)&ImGuiPlatformImeData_ImGuiPlatformImeData, "ImGuiPlatformImeData_ImGuiPlatformImeData");
    lib.bindSymbol(cast(void**)&ImRect_GetBL, "ImRect_GetBL");
    lib.bindSymbol(cast(void**)&igTableGetHeaderRowHeight, "igTableGetHeaderRowHeight");
    lib.bindSymbol(cast(void**)&igIsMousePosValid, "igIsMousePosValid");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetFloat, "ImGuiStorage_GetFloat");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsLeafNode, "ImGuiDockNode_IsLeafNode");
    lib.bindSymbol(cast(void**)&igTableEndCell, "igTableEndCell");
    lib.bindSymbol(cast(void**)&igSliderFloat4, "igSliderFloat4");
    lib.bindSymbol(cast(void**)&igIsItemDeactivatedAfterEdit, "igIsItemDeactivatedAfterEdit");
    lib.bindSymbol(cast(void**)&igPlotHistogram_FloatPtr, "igPlotHistogram_FloatPtr");
    lib.bindSymbol(cast(void**)&igPlotHistogram_FnFloatPtr, "igPlotHistogram_FnFloatPtr");
    lib.bindSymbol(cast(void**)&igIsItemEdited, "igIsItemEdited");
    lib.bindSymbol(cast(void**)&igShowStyleEditor, "igShowStyleEditor");
    lib.bindSymbol(cast(void**)&igTextWrappedV, "igTextWrappedV");
    lib.bindSymbol(cast(void**)&igTableBeginCell, "igTableBeginCell");
    lib.bindSymbol(cast(void**)&igTableGetColumnNextSortDirection, "igTableGetColumnNextSortDirection");
    lib.bindSymbol(cast(void**)&igDebugCheckVersionAndDataLayout, "igDebugCheckVersionAndDataLayout");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_appendf, "ImGuiTextBuffer_appendf");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddCustomRectFontGlyph, "ImFontAtlas_AddCustomRectFontGlyph");
    lib.bindSymbol(cast(void**)&igInputTextWithHint, "igInputTextWithHint");
    lib.bindSymbol(cast(void**)&igImAlphaBlendColors, "igImAlphaBlendColors");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetBoolRef, "ImGuiStorage_GetBoolRef");
    lib.bindSymbol(cast(void**)&igBeginPopupContextVoid, "igBeginPopupContextVoid");
    lib.bindSymbol(cast(void**)&igSetScrollX_Float, "igSetScrollX_Float");
    lib.bindSymbol(cast(void**)&igSetScrollX_WindowPtr, "igSetScrollX_WindowPtr");
    lib.bindSymbol(cast(void**)&igRenderNavHighlight, "igRenderNavHighlight");
    lib.bindSymbol(cast(void**)&igBringWindowToFocusFront, "igBringWindowToFocusFront");
    lib.bindSymbol(cast(void**)&igSliderInt, "igSliderInt");
    lib.bindSymbol(cast(void**)&igUpdateMouseMovingWindowEndFrame, "igUpdateMouseMovingWindowEndFrame");
    lib.bindSymbol(cast(void**)&igSliderInt2, "igSliderInt2");
    lib.bindSymbol(cast(void**)&igGetContentRegionMaxAbs, "igGetContentRegionMaxAbs");
    lib.bindSymbol(cast(void**)&igIsMouseHoveringRect, "igIsMouseHoveringRect");
    lib.bindSymbol(cast(void**)&igImTextStrFromUtf8, "igImTextStrFromUtf8");
    lib.bindSymbol(cast(void**)&igIsActiveIdUsingNavDir, "igIsActiveIdUsingNavDir");
    lib.bindSymbol(cast(void**)&ImGuiListClipper_Begin, "ImGuiListClipper_Begin");
    lib.bindSymbol(cast(void**)&igStartMouseMovingWindow, "igStartMouseMovingWindow");
    lib.bindSymbol(cast(void**)&igIsItemHovered, "igIsItemHovered");
    lib.bindSymbol(cast(void**)&igTableEndRow, "igTableEndRow");
    lib.bindSymbol(cast(void**)&ImGuiIO_destroy, "ImGuiIO_destroy");
    lib.bindSymbol(cast(void**)&igEndDragDropSource, "igEndDragDropSource");
    lib.bindSymbol(cast(void**)&ImGuiStackSizes_SetToCurrentState, "ImGuiStackSizes_SetToCurrentState");
    lib.bindSymbol(cast(void**)&igGetDragDropPayload, "igGetDragDropPayload");
    lib.bindSymbol(cast(void**)&igGetPopupAllowedExtentRect, "igGetPopupAllowedExtentRect");
    lib.bindSymbol(cast(void**)&ImGuiStorage_SetInt, "ImGuiStorage_SetInt");
    lib.bindSymbol(cast(void**)&igIsLegacyKey, "igIsLegacyKey");
    lib.bindSymbol(cast(void**)&ImGuiWindow_MenuBarRect, "ImGuiWindow_MenuBarRect");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetInt, "ImGuiStorage_GetInt");
    lib.bindSymbol(cast(void**)&igShowFontSelector, "igShowFontSelector");
    lib.bindSymbol(cast(void**)&igDestroyPlatformWindow, "igDestroyPlatformWindow");
    lib.bindSymbol(cast(void**)&igImMin, "igImMin");
    lib.bindSymbol(cast(void**)&igCalcRoundingFlagsForRectInRect, "igCalcRoundingFlagsForRectInRect");
    lib.bindSymbol(cast(void**)&igPopButtonRepeat, "igPopButtonRepeat");
    lib.bindSymbol(cast(void**)&igTableSetColumnWidthAutoAll, "igTableSetColumnWidthAutoAll");
    lib.bindSymbol(cast(void**)&igImAbs_Int, "igImAbs_Int");
    lib.bindSymbol(cast(void**)&igImAbs_Float, "igImAbs_Float");
    lib.bindSymbol(cast(void**)&igImAbs_double, "igImAbs_double");
    lib.bindSymbol(cast(void**)&igPushButtonRepeat, "igPushButtonRepeat");
    lib.bindSymbol(cast(void**)&ImGuiWindow_Rect, "ImGuiWindow_Rect");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_GetWorkRect, "ImGuiViewportP_GetWorkRect");
    lib.bindSymbol(cast(void**)&ImRect_Floor, "ImRect_Floor");
    lib.bindSymbol(cast(void**)&igTreePush_Str, "igTreePush_Str");
    lib.bindSymbol(cast(void**)&igTreePush_Ptr, "igTreePush_Ptr");
    lib.bindSymbol(cast(void**)&igColorConvertFloat4ToU32, "igColorConvertFloat4ToU32");
    lib.bindSymbol(cast(void**)&igGetStyle, "igGetStyle");
    lib.bindSymbol(cast(void**)&igGetCursorPos, "igGetCursorPos");
    lib.bindSymbol(cast(void**)&ImGuiStackLevelInfo_ImGuiStackLevelInfo, "ImGuiStackLevelInfo_ImGuiStackLevelInfo");
    lib.bindSymbol(cast(void**)&igGetFrameCount, "igGetFrameCount");
    lib.bindSymbol(cast(void**)&ImDrawList_AddNgon, "ImDrawList_AddNgon");
    lib.bindSymbol(cast(void**)&igDebugNodeDrawList, "igDebugNodeDrawList");
    lib.bindSymbol(cast(void**)&igEnd, "igEnd");
    lib.bindSymbol(cast(void**)&igTabBarCloseTab, "igTabBarCloseTab");
    lib.bindSymbol(cast(void**)&igIsItemActivated, "igIsItemActivated");
    lib.bindSymbol(cast(void**)&igBeginDisabled, "igBeginDisabled");
    lib.bindSymbol(cast(void**)&ImGuiInputEvent_ImGuiInputEvent, "ImGuiInputEvent_ImGuiInputEvent");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_ImGuiInputTextState, "ImGuiInputTextState_ImGuiInputTextState");
    lib.bindSymbol(cast(void**)&ImRect_GetHeight, "ImRect_GetHeight");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddFontDefault, "ImFontAtlas_AddFontDefault");
    lib.bindSymbol(cast(void**)&ImDrawList__OnChangedTextureID, "ImDrawList__OnChangedTextureID");
    lib.bindSymbol(cast(void**)&igGetColumnsCount, "igGetColumnsCount");
    lib.bindSymbol(cast(void**)&igEndChild, "igEndChild");
    lib.bindSymbol(cast(void**)&igNavMoveRequestButNoResultYet, "igNavMoveRequestButNoResultYet");
    lib.bindSymbol(cast(void**)&ImGuiPlatformImeData_destroy, "ImGuiPlatformImeData_destroy");
    lib.bindSymbol(cast(void**)&ImGuiStyle_ScaleAllSizes, "ImGuiStyle_ScaleAllSizes");
    lib.bindSymbol(cast(void**)&igArrowButton, "igArrowButton");
    lib.bindSymbol(cast(void**)&igSetCursorPosY, "igSetCursorPosY");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsFloatingNode, "ImGuiDockNode_IsFloatingNode");
    lib.bindSymbol(cast(void**)&ImGuiTextFilter_ImGuiTextFilter, "ImGuiTextFilter_ImGuiTextFilter");
    lib.bindSymbol(cast(void**)&ImGuiStorage_SetFloat, "ImGuiStorage_SetFloat");
    lib.bindSymbol(cast(void**)&igShadeVertsLinearUV, "igShadeVertsLinearUV");
    lib.bindSymbol(cast(void**)&igTableGetColumnIndex, "igTableGetColumnIndex");
    lib.bindSymbol(cast(void**)&igGetTime, "igGetTime");
    lib.bindSymbol(cast(void**)&igBeginPopupContextItem, "igBeginPopupContextItem");
    lib.bindSymbol(cast(void**)&igImRsqrt_Float, "igImRsqrt_Float");
    lib.bindSymbol(cast(void**)&igImRsqrt_double, "igImRsqrt_double");
    lib.bindSymbol(cast(void**)&igTableLoadSettings, "igTableLoadSettings");
    lib.bindSymbol(cast(void**)&igSetScrollHereX, "igSetScrollHereX");
    lib.bindSymbol(cast(void**)&igSliderScalarN, "igSliderScalarN");
    lib.bindSymbol(cast(void**)&ImDrawList_PathBezierQuadraticCurveTo, "ImDrawList_PathBezierQuadraticCurveTo");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesChineseSimplifiedCommon, "ImFontAtlas_GetGlyphRangesChineseSimplifiedCommon");
    lib.bindSymbol(cast(void**)&igGetMousePosOnOpeningCurrentPopup, "igGetMousePosOnOpeningCurrentPopup");
    lib.bindSymbol(cast(void**)&igVSliderScalar, "igVSliderScalar");
    lib.bindSymbol(cast(void**)&igDockBuilderSetNodePos, "igDockBuilderSetNodePos");
    lib.bindSymbol(cast(void**)&ImFont_RenderChar, "ImFont_RenderChar");
    lib.bindSymbol(cast(void**)&ImFont_RenderText, "ImFont_RenderText");
    lib.bindSymbol(cast(void**)&igOpenPopupEx, "igOpenPopupEx");
    lib.bindSymbol(cast(void**)&ImFontAtlas_SetTexID, "ImFontAtlas_SetTexID");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildRender8bppRectFromString, "igImFontAtlasBuildRender8bppRectFromString");
    lib.bindSymbol(cast(void**)&ImFontAtlas_Clear, "ImFontAtlas_Clear");
    lib.bindSymbol(cast(void**)&igBeginDockableDragDropSource, "igBeginDockableDragDropSource");
    lib.bindSymbol(cast(void**)&ImBitVector_TestBit, "ImBitVector_TestBit");
    lib.bindSymbol(cast(void**)&igGetMergedModFlags, "igGetMergedModFlags");
    lib.bindSymbol(cast(void**)&ImGuiTextFilter_destroy, "ImGuiTextFilter_destroy");
    lib.bindSymbol(cast(void**)&igBeginPopupModal, "igBeginPopupModal");
    lib.bindSymbol(cast(void**)&igGetFocusedFocusScope, "igGetFocusedFocusScope");
    lib.bindSymbol(cast(void**)&igGetMouseClickedCount, "igGetMouseClickedCount");
    lib.bindSymbol(cast(void**)&igDebugNodeColumns, "igDebugNodeColumns");
    lib.bindSymbol(cast(void**)&igDebugNodeWindow, "igDebugNodeWindow");
    lib.bindSymbol(cast(void**)&igGetWindowDpiScale, "igGetWindowDpiScale");
    lib.bindSymbol(cast(void**)&igInputFloat, "igInputFloat");
    lib.bindSymbol(cast(void**)&igDragIntRange2, "igDragIntRange2");
    lib.bindSymbol(cast(void**)&ImVec2ih_destroy, "ImVec2ih_destroy");
    lib.bindSymbol(cast(void**)&ImDrawList_GetClipRectMax, "ImDrawList_GetClipRectMax");
    lib.bindSymbol(cast(void**)&igInputFloat2, "igInputFloat2");
    lib.bindSymbol(cast(void**)&ImDrawDataBuilder_ClearFreeMemory, "ImDrawDataBuilder_ClearFreeMemory");
    lib.bindSymbol(cast(void**)&ImGuiLastItemData_destroy, "ImGuiLastItemData_destroy");
    lib.bindSymbol(cast(void**)&ImGuiWindowSettings_GetName, "ImGuiWindowSettings_GetName");
    lib.bindSymbol(cast(void**)&igImStrdup, "igImStrdup");
    lib.bindSymbol(cast(void**)&igImFormatStringV, "igImFormatStringV");
    lib.bindSymbol(cast(void**)&igSetTooltipV, "igSetTooltipV");
    lib.bindSymbol(cast(void**)&igDataTypeGetInfo, "igDataTypeGetInfo");
    lib.bindSymbol(cast(void**)&igVSliderInt, "igVSliderInt");
    lib.bindSymbol(cast(void**)&igSetWindowClipRectBeforeSetChannel, "igSetWindowClipRectBeforeSetChannel");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_ImFontGlyphRangesBuilder, "ImFontGlyphRangesBuilder_ImFontGlyphRangesBuilder");
    lib.bindSymbol(cast(void**)&igGetWindowDockID, "igGetWindowDockID");
    lib.bindSymbol(cast(void**)&igPopTextWrapPos, "igPopTextWrapPos");
    lib.bindSymbol(cast(void**)&ImGuiWindowClass_destroy, "ImGuiWindowClass_destroy");
    lib.bindSymbol(cast(void**)&ImGuiWindow_TitleBarHeight, "ImGuiWindow_TitleBarHeight");
    lib.bindSymbol(cast(void**)&ImDrawList_GetClipRectMin, "ImDrawList_GetClipRectMin");
    lib.bindSymbol(cast(void**)&ImDrawList_PathStroke, "ImDrawList_PathStroke");
    lib.bindSymbol(cast(void**)&igBeginTooltip, "igBeginTooltip");
    lib.bindSymbol(cast(void**)&igOpenPopupOnItemClick, "igOpenPopupOnItemClick");
    lib.bindSymbol(cast(void**)&ImDrawListSplitter_Merge, "ImDrawListSplitter_Merge");
    lib.bindSymbol(cast(void**)&ImGuiWindow_MenuBarHeight, "ImGuiWindow_MenuBarHeight");
    lib.bindSymbol(cast(void**)&ImColor_HSV, "ImColor_HSV");
    lib.bindSymbol(cast(void**)&igBeginTableEx, "igBeginTableEx");
    lib.bindSymbol(cast(void**)&igSetTabItemClosed, "igSetTabItemClosed");
    lib.bindSymbol(cast(void**)&ImFont_AddGlyph, "ImFont_AddGlyph");
    lib.bindSymbol(cast(void**)&igSetHoveredID, "igSetHoveredID");
    lib.bindSymbol(cast(void**)&igStartMouseMovingWindowOrNode, "igStartMouseMovingWindowOrNode");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_AddText, "ImFontGlyphRangesBuilder_AddText");
    lib.bindSymbol(cast(void**)&ImGuiPtrOrIndex_destroy, "ImGuiPtrOrIndex_destroy");
    lib.bindSymbol(cast(void**)&ImGuiInputTextCallbackData_ImGuiInputTextCallbackData, "ImGuiInputTextCallbackData_ImGuiInputTextCallbackData");
    lib.bindSymbol(cast(void**)&igImStrdupcpy, "igImStrdupcpy");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsNoTabBar, "ImGuiDockNode_IsNoTabBar");
    lib.bindSymbol(cast(void**)&igColorConvertHSVtoRGB, "igColorConvertHSVtoRGB");
    lib.bindSymbol(cast(void**)&igDockBuilderSplitNode, "igDockBuilderSplitNode");
    lib.bindSymbol(cast(void**)&igColorPicker4, "igColorPicker4");
    lib.bindSymbol(cast(void**)&igImBitArrayTestBit, "igImBitArrayTestBit");
    lib.bindSymbol(cast(void**)&igFindWindowByID, "igFindWindowByID");
    lib.bindSymbol(cast(void**)&ImDrawList_PathBezierCubicCurveTo, "ImDrawList_PathBezierCubicCurveTo");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddMouseWheelEvent, "ImGuiIO_AddMouseWheelEvent");
    lib.bindSymbol(cast(void**)&igBeginDragDropTargetCustom, "igBeginDragDropTargetCustom");
    lib.bindSymbol(cast(void**)&ImGuiContext_destroy, "ImGuiContext_destroy");
    lib.bindSymbol(cast(void**)&igDragInt3, "igDragInt3");
    lib.bindSymbol(cast(void**)&igImHashStr, "igImHashStr");
    lib.bindSymbol(cast(void**)&ImDrawList_AddTriangleFilled, "ImDrawList_AddTriangleFilled");
    lib.bindSymbol(cast(void**)&igDebugNodeFont, "igDebugNodeFont");
    lib.bindSymbol(cast(void**)&igRenderArrow, "igRenderArrow");
    lib.bindSymbol(cast(void**)&igNewFrame, "igNewFrame");
    lib.bindSymbol(cast(void**)&ImGuiTabItem_ImGuiTabItem, "ImGuiTabItem_ImGuiTabItem");
    lib.bindSymbol(cast(void**)&ImDrawList_ChannelsSetCurrent, "ImDrawList_ChannelsSetCurrent");
    lib.bindSymbol(cast(void**)&igClosePopupToLevel, "igClosePopupToLevel");
    lib.bindSymbol(cast(void**)&ImGuiContext_ImGuiContext, "ImGuiContext_ImGuiContext");
    lib.bindSymbol(cast(void**)&igSliderFloat2, "igSliderFloat2");
    lib.bindSymbol(cast(void**)&igTempInputScalar, "igTempInputScalar");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddKeyAnalogEvent, "ImGuiIO_AddKeyAnalogEvent");
    lib.bindSymbol(cast(void**)&ImGuiPopupData_ImGuiPopupData, "ImGuiPopupData_ImGuiPopupData");
    lib.bindSymbol(cast(void**)&ImDrawList_AddImageQuad, "ImDrawList_AddImageQuad");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetCustomRectByIndex, "ImFontAtlas_GetCustomRectByIndex");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetTexDataAsAlpha8, "ImFontAtlas_GetTexDataAsAlpha8");
    lib.bindSymbol(cast(void**)&igBeginListBox, "igBeginListBox");
    lib.bindSymbol(cast(void**)&igGcAwakeTransientWindowBuffers, "igGcAwakeTransientWindowBuffers");
    lib.bindSymbol(cast(void**)&ImDrawList__OnChangedClipRect, "ImDrawList__OnChangedClipRect");
    lib.bindSymbol(cast(void**)&ImGuiWindowClass_ImGuiWindowClass, "ImGuiWindowClass_ImGuiWindowClass");
    lib.bindSymbol(cast(void**)&igDockBuilderRemoveNodeChildNodes, "igDockBuilderRemoveNodeChildNodes");
    lib.bindSymbol(cast(void**)&igGetColumnsID, "igGetColumnsID");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_SetLocalFlags, "ImGuiDockNode_SetLocalFlags");
    lib.bindSymbol(cast(void**)&igPushAllowKeyboardFocus, "igPushAllowKeyboardFocus");
    lib.bindSymbol(cast(void**)&ImDrawList_PopTextureID, "ImDrawList_PopTextureID");
    lib.bindSymbol(cast(void**)&igColumns, "igColumns");
    lib.bindSymbol(cast(void**)&ImGuiListClipperRange_FromPositions, "ImGuiListClipperRange_FromPositions");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_AddChar, "ImFontGlyphRangesBuilder_AddChar");
    lib.bindSymbol(cast(void**)&igNavMoveRequestResolveWithLastItem, "igNavMoveRequestResolveWithLastItem");
    lib.bindSymbol(cast(void**)&igGetColumnIndex, "igGetColumnIndex");
    lib.bindSymbol(cast(void**)&igBringWindowToDisplayBack, "igBringWindowToDisplayBack");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimVtx, "ImDrawList_PrimVtx");
    lib.bindSymbol(cast(void**)&ImDrawListSplitter_Clear, "ImDrawListSplitter_Clear");
    lib.bindSymbol(cast(void**)&igImTextCharToUtf8, "igImTextCharToUtf8");
    lib.bindSymbol(cast(void**)&igTableBeginInitMemory, "igTableBeginInitMemory");
    lib.bindSymbol(cast(void**)&ImDrawList_AddConvexPolyFilled, "ImDrawList_AddConvexPolyFilled");
    lib.bindSymbol(cast(void**)&igGetCursorScreenPos, "igGetCursorScreenPos");
    lib.bindSymbol(cast(void**)&igListBox_Str_arr, "igListBox_Str_arr");
    lib.bindSymbol(cast(void**)&igListBox_FnBoolPtr, "igListBox_FnBoolPtr");
    lib.bindSymbol(cast(void**)&igPopItemFlag, "igPopItemFlag");
    lib.bindSymbol(cast(void**)&igImBezierCubicClosestPoint, "igImBezierCubicClosestPoint");
    lib.bindSymbol(cast(void**)&igGetItemFlags, "igGetItemFlags");
    lib.bindSymbol(cast(void**)&igPopColumnsBackground, "igPopColumnsBackground");
    lib.bindSymbol(cast(void**)&igLogBegin, "igLogBegin");
    lib.bindSymbol(cast(void**)&igTreeNodeV_Str, "igTreeNodeV_Str");
    lib.bindSymbol(cast(void**)&igTreeNodeV_Ptr, "igTreeNodeV_Ptr");
    lib.bindSymbol(cast(void**)&igRenderTextClippedEx, "igRenderTextClippedEx");
    lib.bindSymbol(cast(void**)&igTableSettingsFindByID, "igTableSettingsFindByID");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddInputCharacterUTF16, "ImGuiIO_AddInputCharacterUTF16");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetFloatRef, "ImGuiStorage_GetFloatRef");
    lib.bindSymbol(cast(void**)&igImStrbolW, "igImStrbolW");
    lib.bindSymbol(cast(void**)&ImGuiStackSizes_ImGuiStackSizes, "ImGuiStackSizes_ImGuiStackSizes");
    lib.bindSymbol(cast(void**)&igSliderBehavior, "igSliderBehavior");
    lib.bindSymbol(cast(void**)&igValue_Bool, "igValue_Bool");
    lib.bindSymbol(cast(void**)&igValue_Int, "igValue_Int");
    lib.bindSymbol(cast(void**)&igValue_Uint, "igValue_Uint");
    lib.bindSymbol(cast(void**)&igValue_Float, "igValue_Float");
    lib.bindSymbol(cast(void**)&igBeginTabItem, "igBeginTabItem");
    lib.bindSymbol(cast(void**)&igDebugNodeTable, "igDebugNodeTable");
    lib.bindSymbol(cast(void**)&ImGuiViewport_destroy, "ImGuiViewport_destroy");
    lib.bindSymbol(cast(void**)&igIsNavInputDown, "igIsNavInputDown");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_ClearFreeMemory, "ImGuiInputTextState_ClearFreeMemory");
    lib.bindSymbol(cast(void**)&ImGuiViewport_GetWorkCenter, "ImGuiViewport_GetWorkCenter");
    lib.bindSymbol(cast(void**)&igRenderBullet, "igRenderBullet");
    lib.bindSymbol(cast(void**)&igDragFloat4, "igDragFloat4");
    lib.bindSymbol(cast(void**)&igWindowRectAbsToRel, "igWindowRectAbsToRel");
    lib.bindSymbol(cast(void**)&ImDrawList__OnChangedVtxOffset, "ImDrawList__OnChangedVtxOffset");
    lib.bindSymbol(cast(void**)&igTableSortSpecsSanitize, "igTableSortSpecsSanitize");
    lib.bindSymbol(cast(void**)&ImGuiInputEvent_destroy, "ImGuiInputEvent_destroy");
    lib.bindSymbol(cast(void**)&igFocusTopMostWindowUnderOne, "igFocusTopMostWindowUnderOne");
    lib.bindSymbol(cast(void**)&igPushID_Str, "igPushID_Str");
    lib.bindSymbol(cast(void**)&igPushID_StrStr, "igPushID_StrStr");
    lib.bindSymbol(cast(void**)&igPushID_Ptr, "igPushID_Ptr");
    lib.bindSymbol(cast(void**)&igPushID_Int, "igPushID_Int");
    lib.bindSymbol(cast(void**)&igItemHoverable, "igItemHoverable");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddFontFromMemoryTTF, "ImFontAtlas_AddFontFromMemoryTTF");
    lib.bindSymbol(cast(void**)&igDockBuilderDockWindow, "igDockBuilderDockWindow");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildMultiplyRectAlpha8, "igImFontAtlasBuildMultiplyRectAlpha8");
    lib.bindSymbol(cast(void**)&igTextDisabledV, "igTextDisabledV");
    lib.bindSymbol(cast(void**)&igInputScalar, "igInputScalar");
    lib.bindSymbol(cast(void**)&ImGuiPtrOrIndex_ImGuiPtrOrIndex_Ptr, "ImGuiPtrOrIndex_ImGuiPtrOrIndex_Ptr");
    lib.bindSymbol(cast(void**)&ImGuiPtrOrIndex_ImGuiPtrOrIndex_Int, "ImGuiPtrOrIndex_ImGuiPtrOrIndex_Int");
    lib.bindSymbol(cast(void**)&igImLerp_Vec2Float, "igImLerp_Vec2Float");
    lib.bindSymbol(cast(void**)&igImLerp_Vec2Vec2, "igImLerp_Vec2Vec2");
    lib.bindSymbol(cast(void**)&igImLerp_Vec4, "igImLerp_Vec4");
    lib.bindSymbol(cast(void**)&igItemSize_Vec2, "igItemSize_Vec2");
    lib.bindSymbol(cast(void**)&igItemSize_Rect, "igItemSize_Rect");
    lib.bindSymbol(cast(void**)&ImColor_SetHSV, "ImColor_SetHSV");
    lib.bindSymbol(cast(void**)&ImFont_IsGlyphRangeUnused, "ImFont_IsGlyphRangeUnused");
    lib.bindSymbol(cast(void**)&igImBezierQuadraticCalc, "igImBezierQuadraticCalc");
    lib.bindSymbol(cast(void**)&igImParseFormatPrecision, "igImParseFormatPrecision");
    lib.bindSymbol(cast(void**)&igLogToTTY, "igLogToTTY");
    lib.bindSymbol(cast(void**)&igTableGetColumnWidthAuto, "igTableGetColumnWidthAuto");
    lib.bindSymbol(cast(void**)&igButtonBehavior, "igButtonBehavior");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_OnKeyPressed, "ImGuiInputTextState_OnKeyPressed");
    lib.bindSymbol(cast(void**)&igImLog_Float, "igImLog_Float");
    lib.bindSymbol(cast(void**)&igImLog_double, "igImLog_double");
    lib.bindSymbol(cast(void**)&igSetFocusID, "igSetFocusID");
    lib.bindSymbol(cast(void**)&igGetActiveID, "igGetActiveID");
    lib.bindSymbol(cast(void**)&igImLineClosestPoint, "igImLineClosestPoint");
    lib.bindSymbol(cast(void**)&igDebugNodeWindowsListByBeginStackParent, "igDebugNodeWindowsListByBeginStackParent");
    lib.bindSymbol(cast(void**)&igIsItemVisible, "igIsItemVisible");
    lib.bindSymbol(cast(void**)&igRender, "igRender");
    lib.bindSymbol(cast(void**)&igImTriangleArea, "igImTriangleArea");
    lib.bindSymbol(cast(void**)&igBeginChild_Str, "igBeginChild_Str");
    lib.bindSymbol(cast(void**)&igBeginChild_ID, "igBeginChild_ID");
    lib.bindSymbol(cast(void**)&igStyleColorsLight, "igStyleColorsLight");
    lib.bindSymbol(cast(void**)&igGetScrollX, "igGetScrollX");
    lib.bindSymbol(cast(void**)&igCallContextHooks, "igCallContextHooks");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetTexDataAsRGBA32, "ImFontAtlas_GetTexDataAsRGBA32");
    lib.bindSymbol(cast(void**)&ImGuiOnceUponAFrame_ImGuiOnceUponAFrame, "ImGuiOnceUponAFrame_ImGuiOnceUponAFrame");
    lib.bindSymbol(cast(void**)&ImDrawData_destroy, "ImDrawData_destroy");
    lib.bindSymbol(cast(void**)&igSaveIniSettingsToMemory, "igSaveIniSettingsToMemory");
    lib.bindSymbol(cast(void**)&igTabBarRemoveTab, "igTabBarRemoveTab");
    lib.bindSymbol(cast(void**)&igGetWindowHeight, "igGetWindowHeight");
    lib.bindSymbol(cast(void**)&igGetMainViewport, "igGetMainViewport");
    lib.bindSymbol(cast(void**)&ImGuiTextFilter_PassFilter, "ImGuiTextFilter_PassFilter");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddFontFromMemoryCompressedBase85TTF, "ImFontAtlas_AddFontFromMemoryCompressedBase85TTF");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddFontFromFileTTF, "ImFontAtlas_AddFontFromFileTTF");
    lib.bindSymbol(cast(void**)&igEndDisabled, "igEndDisabled");
    lib.bindSymbol(cast(void**)&ImGuiViewportP_CalcWorkRectSize, "ImGuiViewportP_CalcWorkRectSize");
    lib.bindSymbol(cast(void**)&igGetCurrentContext, "igGetCurrentContext");
    lib.bindSymbol(cast(void**)&igColorConvertU32ToFloat4, "igColorConvertU32ToFloat4");
    lib.bindSymbol(cast(void**)&ImDrawList_PathArcToFast, "ImDrawList_PathArcToFast");
    lib.bindSymbol(cast(void**)&igDragFloat, "igDragFloat");
    lib.bindSymbol(cast(void**)&igGetStyleColorName, "igGetStyleColorName");
    lib.bindSymbol(cast(void**)&igSetItemDefaultFocus, "igSetItemDefaultFocus");
    lib.bindSymbol(cast(void**)&ImGuiDockNodeSettings_ImGuiDockNodeSettings, "ImGuiDockNodeSettings_ImGuiDockNodeSettings");
    lib.bindSymbol(cast(void**)&igSetNextWindowPos, "igSetNextWindowPos");
    lib.bindSymbol(cast(void**)&igDragFloat3, "igDragFloat3");
    lib.bindSymbol(cast(void**)&igCaptureKeyboardFromApp, "igCaptureKeyboardFromApp");
    lib.bindSymbol(cast(void**)&igGetCurrentTable, "igGetCurrentTable");
    lib.bindSymbol(cast(void**)&ImDrawData_Clear, "ImDrawData_Clear");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddFontFromMemoryCompressedTTF, "ImFontAtlas_AddFontFromMemoryCompressedTTF");
    lib.bindSymbol(cast(void**)&ImGuiStoragePair_destroy, "ImGuiStoragePair_destroy");
    lib.bindSymbol(cast(void**)&igIsItemToggledOpen, "igIsItemToggledOpen");
    lib.bindSymbol(cast(void**)&igInputInt3, "igInputInt3");
    lib.bindSymbol(cast(void**)&igShrinkWidths, "igShrinkWidths");
    lib.bindSymbol(cast(void**)&igClosePopupsExceptModals, "igClosePopupsExceptModals");
    lib.bindSymbol(cast(void**)&ImDrawList_AddText_Vec2, "ImDrawList_AddText_Vec2");
    lib.bindSymbol(cast(void**)&ImDrawList_AddText_FontPtr, "ImDrawList_AddText_FontPtr");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimRectUV, "ImDrawList_PrimRectUV");
    lib.bindSymbol(cast(void**)&igFindBottomMostVisibleWindowWithinBeginStack, "igFindBottomMostVisibleWindowWithinBeginStack");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimWriteIdx, "ImDrawList_PrimWriteIdx");
    lib.bindSymbol(cast(void**)&ImGuiOldColumns_ImGuiOldColumns, "ImGuiOldColumns_ImGuiOldColumns");
    lib.bindSymbol(cast(void**)&igTableRemove, "igTableRemove");
    lib.bindSymbol(cast(void**)&igDebugNodeTableSettings, "igDebugNodeTableSettings");
    lib.bindSymbol(cast(void**)&ImGuiStorage_GetBool, "ImGuiStorage_GetBool");
    lib.bindSymbol(cast(void**)&igRenderFrameBorder, "igRenderFrameBorder");
    lib.bindSymbol(cast(void**)&igFindWindowByName, "igFindWindowByName");
    lib.bindSymbol(cast(void**)&ImGuiLastItemData_ImGuiLastItemData, "ImGuiLastItemData_ImGuiLastItemData");
    lib.bindSymbol(cast(void**)&igImTextStrToUtf8, "igImTextStrToUtf8");
    lib.bindSymbol(cast(void**)&igScrollToBringRectIntoView, "igScrollToBringRectIntoView");
    lib.bindSymbol(cast(void**)&igTableSettingsAddSettingsHandler, "igTableSettingsAddSettingsHandler");
    lib.bindSymbol(cast(void**)&igInputInt, "igInputInt");
    lib.bindSymbol(cast(void**)&ImVec2_ImVec2_Nil, "ImVec2_ImVec2_Nil");
    lib.bindSymbol(cast(void**)&ImVec2_ImVec2_Float, "ImVec2_ImVec2_Float");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_size, "ImGuiTextBuffer_size");
    lib.bindSymbol(cast(void**)&ImFontAtlas_GetGlyphRangesDefault, "ImFontAtlas_GetGlyphRangesDefault");
    lib.bindSymbol(cast(void**)&igUpdatePlatformWindows, "igUpdatePlatformWindows");
    lib.bindSymbol(cast(void**)&igTextWrapped, "igTextWrapped");
    lib.bindSymbol(cast(void**)&ImFontAtlas_ClearTexData, "ImFontAtlas_ClearTexData");
    lib.bindSymbol(cast(void**)&ImFont_GetCharAdvance, "ImFont_GetCharAdvance");
    lib.bindSymbol(cast(void**)&igSliderFloat3, "igSliderFloat3");
    lib.bindSymbol(cast(void**)&ImDrawList_PathFillConvex, "ImDrawList_PathFillConvex");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_ImGuiTextBuffer, "ImGuiTextBuffer_ImGuiTextBuffer");
    lib.bindSymbol(cast(void**)&ImGuiTabItem_destroy, "ImGuiTabItem_destroy");
    lib.bindSymbol(cast(void**)&igSliderAngle, "igSliderAngle");
    lib.bindSymbol(cast(void**)&ImGuiTableColumnSortSpecs_destroy, "ImGuiTableColumnSortSpecs_destroy");
    lib.bindSymbol(cast(void**)&igSetWindowPos_Vec2, "igSetWindowPos_Vec2");
    lib.bindSymbol(cast(void**)&igSetWindowPos_Str, "igSetWindowPos_Str");
    lib.bindSymbol(cast(void**)&igSetWindowPos_WindowPtr, "igSetWindowPos_WindowPtr");
    lib.bindSymbol(cast(void**)&igTempInputText, "igTempInputText");
    lib.bindSymbol(cast(void**)&igSetScrollHereY, "igSetScrollHereY");
    lib.bindSymbol(cast(void**)&igMenuItemEx, "igMenuItemEx");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddFocusEvent, "ImGuiIO_AddFocusEvent");
    lib.bindSymbol(cast(void**)&ImGuiViewport_ImGuiViewport, "ImGuiViewport_ImGuiViewport");
    lib.bindSymbol(cast(void**)&igProgressBar, "igProgressBar");
    lib.bindSymbol(cast(void**)&ImDrawList_CloneOutput, "ImDrawList_CloneOutput");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_destroy, "ImFontGlyphRangesBuilder_destroy");
    lib.bindSymbol(cast(void**)&ImVec1_destroy, "ImVec1_destroy");
    lib.bindSymbol(cast(void**)&igPushColumnClipRect, "igPushColumnClipRect");
    lib.bindSymbol(cast(void**)&igTabBarQueueReorderFromMousePos, "igTabBarQueueReorderFromMousePos");
    lib.bindSymbol(cast(void**)&igLogTextV, "igLogTextV");
    lib.bindSymbol(cast(void**)&igDockBuilderCopyWindowSettings, "igDockBuilderCopyWindowSettings");
    lib.bindSymbol(cast(void**)&igImTextCharFromUtf8, "igImTextCharFromUtf8");
    lib.bindSymbol(cast(void**)&ImRect_ImRect_Nil, "ImRect_ImRect_Nil");
    lib.bindSymbol(cast(void**)&ImRect_ImRect_Vec2, "ImRect_ImRect_Vec2");
    lib.bindSymbol(cast(void**)&ImRect_ImRect_Vec4, "ImRect_ImRect_Vec4");
    lib.bindSymbol(cast(void**)&ImRect_ImRect_Float, "ImRect_ImRect_Float");
    lib.bindSymbol(cast(void**)&igGetTopMostPopupModal, "igGetTopMostPopupModal");
    lib.bindSymbol(cast(void**)&ImDrawListSplitter_Split, "ImDrawListSplitter_Split");
    lib.bindSymbol(cast(void**)&igBulletText, "igBulletText");
    lib.bindSymbol(cast(void**)&igImFontAtlasBuildFinish, "igImFontAtlasBuildFinish");
    lib.bindSymbol(cast(void**)&igDebugNodeViewport, "igDebugNodeViewport");
    lib.bindSymbol(cast(void**)&ImDrawList_AddQuad, "ImDrawList_AddQuad");
    lib.bindSymbol(cast(void**)&igDockSpace, "igDockSpace");
    lib.bindSymbol(cast(void**)&igGetColorU32_Col, "igGetColorU32_Col");
    lib.bindSymbol(cast(void**)&igGetColorU32_Vec4, "igGetColorU32_Vec4");
    lib.bindSymbol(cast(void**)&igGetColorU32_U32, "igGetColorU32_U32");
    lib.bindSymbol(cast(void**)&ImGuiWindow_GetIDFromRectangle, "ImGuiWindow_GetIDFromRectangle");
    lib.bindSymbol(cast(void**)&ImDrawList_AddDrawCmd, "ImDrawList_AddDrawCmd");
    lib.bindSymbol(cast(void**)&igDebugNodeFontGlyph, "igDebugNodeFontGlyph");
    lib.bindSymbol(cast(void**)&igIsItemDeactivated, "igIsItemDeactivated");
    lib.bindSymbol(cast(void**)&igSetCursorPosX, "igSetCursorPosX");
    lib.bindSymbol(cast(void**)&igInputFloat4, "igInputFloat4");
    lib.bindSymbol(cast(void**)&igSeparator, "igSeparator");
    lib.bindSymbol(cast(void**)&ImRect_Translate, "ImRect_Translate");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimUnreserve, "ImDrawList_PrimUnreserve");
    lib.bindSymbol(cast(void**)&igUpdateWindowParentAndRootLinks, "igUpdateWindowParentAndRootLinks");
    lib.bindSymbol(cast(void**)&igColorPickerOptionsPopup, "igColorPickerOptionsPopup");
    lib.bindSymbol(cast(void**)&ImRect_IsInverted, "ImRect_IsInverted");
    lib.bindSymbol(cast(void**)&igGetKeyIndex, "igGetKeyIndex");
    lib.bindSymbol(cast(void**)&igFindViewportByID, "igFindViewportByID");
    lib.bindSymbol(cast(void**)&ImGuiMetricsConfig_destroy, "ImGuiMetricsConfig_destroy");
    lib.bindSymbol(cast(void**)&igPushItemFlag, "igPushItemFlag");
    lib.bindSymbol(cast(void**)&igScrollbar, "igScrollbar");
    lib.bindSymbol(cast(void**)&igDebugNodeWindowsList, "igDebugNodeWindowsList");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimWriteVtx, "ImDrawList_PrimWriteVtx");
    lib.bindSymbol(cast(void**)&ImGuiDockContext_destroy, "ImGuiDockContext_destroy");
    lib.bindSymbol(cast(void**)&ImGuiPayload_IsDataType, "ImGuiPayload_IsDataType");
    lib.bindSymbol(cast(void**)&igSetActiveID, "igSetActiveID");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_BuildRanges, "ImFontGlyphRangesBuilder_BuildRanges");
    lib.bindSymbol(cast(void**)&ImGuiDockPreviewData_ImGuiDockPreviewData, "ImGuiDockPreviewData_ImGuiDockPreviewData");
    lib.bindSymbol(cast(void**)&igScrollToItem, "igScrollToItem");
    lib.bindSymbol(cast(void**)&igTreePop, "igTreePop");
    lib.bindSymbol(cast(void**)&igSetWindowSize_Vec2, "igSetWindowSize_Vec2");
    lib.bindSymbol(cast(void**)&igSetWindowSize_Str, "igSetWindowSize_Str");
    lib.bindSymbol(cast(void**)&igSetWindowSize_WindowPtr, "igSetWindowSize_WindowPtr");
    lib.bindSymbol(cast(void**)&igTableGetCellBgRect, "igTableGetCellBgRect");
    lib.bindSymbol(cast(void**)&ImGuiIO_AddMouseButtonEvent, "ImGuiIO_AddMouseButtonEvent");
    lib.bindSymbol(cast(void**)&ImFont_AddRemapChar, "ImFont_AddRemapChar");
    lib.bindSymbol(cast(void**)&igNavMoveRequestCancel, "igNavMoveRequestCancel");
    lib.bindSymbol(cast(void**)&igText, "igText");
    lib.bindSymbol(cast(void**)&igCollapseButton, "igCollapseButton");
    lib.bindSymbol(cast(void**)&ImGuiStackLevelInfo_destroy, "ImGuiStackLevelInfo_destroy");
    lib.bindSymbol(cast(void**)&ImGuiWindow_TitleBarRect, "ImGuiWindow_TitleBarRect");
    lib.bindSymbol(cast(void**)&igIsItemFocused, "igIsItemFocused");
    lib.bindSymbol(cast(void**)&igGetKeyData, "igGetKeyData");
    lib.bindSymbol(cast(void**)&igMemAlloc, "igMemAlloc");
    lib.bindSymbol(cast(void**)&igTranslateWindowsInViewport, "igTranslateWindowsInViewport");
    lib.bindSymbol(cast(void**)&ImGuiStackSizes_destroy, "ImGuiStackSizes_destroy");
    lib.bindSymbol(cast(void**)&igColorPicker3, "igColorPicker3");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_destroy, "ImGuiTextBuffer_destroy");
    lib.bindSymbol(cast(void**)&igGetColumnOffset, "igGetColumnOffset");
    lib.bindSymbol(cast(void**)&igSetCurrentViewport, "igSetCurrentViewport");
    lib.bindSymbol(cast(void**)&ImRect_GetSize, "ImRect_GetSize");
    lib.bindSymbol(cast(void**)&igSetItemUsingMouseWheel, "igSetItemUsingMouseWheel");
    lib.bindSymbol(cast(void**)&igIsWindowCollapsed, "igIsWindowCollapsed");
    lib.bindSymbol(cast(void**)&ImGuiNextItemData_ClearFlags, "ImGuiNextItemData_ClearFlags");
    lib.bindSymbol(cast(void**)&igBeginCombo, "igBeginCombo");
    lib.bindSymbol(cast(void**)&ImRect_Expand_Float, "ImRect_Expand_Float");
    lib.bindSymbol(cast(void**)&ImRect_Expand_Vec2, "ImRect_Expand_Vec2");
    lib.bindSymbol(cast(void**)&igNavMoveRequestApplyResult, "igNavMoveRequestApplyResult");
    lib.bindSymbol(cast(void**)&igOpenPopup_Str, "igOpenPopup_Str");
    lib.bindSymbol(cast(void**)&igOpenPopup_ID, "igOpenPopup_ID");
    lib.bindSymbol(cast(void**)&igImCharIsBlankW, "igImCharIsBlankW");
    lib.bindSymbol(cast(void**)&ImFont_SetGlyphVisible, "ImFont_SetGlyphVisible");
    lib.bindSymbol(cast(void**)&igFindOrCreateWindowSettings, "igFindOrCreateWindowSettings");
    lib.bindSymbol(cast(void**)&igImFloorSigned_Float, "igImFloorSigned_Float");
    lib.bindSymbol(cast(void**)&igImFloorSigned_Vec2, "igImFloorSigned_Vec2");
    lib.bindSymbol(cast(void**)&igInputScalarN, "igInputScalarN");
    lib.bindSymbol(cast(void**)&ImDrawList_PrimQuadUV, "ImDrawList_PrimQuadUV");
    lib.bindSymbol(cast(void**)&igPopID, "igPopID");
    lib.bindSymbol(cast(void**)&igEndTabBar, "igEndTabBar");
    lib.bindSymbol(cast(void**)&igPopAllowKeyboardFocus, "igPopAllowKeyboardFocus");
    lib.bindSymbol(cast(void**)&ImDrawList_AddImage, "ImDrawList_AddImage");
    lib.bindSymbol(cast(void**)&igBeginTabBar, "igBeginTabBar");
    lib.bindSymbol(cast(void**)&igGetCursorPosY, "igGetCursorPosY");
    lib.bindSymbol(cast(void**)&igCalcTextSize, "igCalcTextSize");
    lib.bindSymbol(cast(void**)&igSetActiveIdUsingNavAndKeys, "igSetActiveIdUsingNavAndKeys");
    lib.bindSymbol(cast(void**)&ImFont_CalcTextSizeA, "ImFont_CalcTextSizeA");
    lib.bindSymbol(cast(void**)&igImClamp, "igImClamp");
    lib.bindSymbol(cast(void**)&igGetColumnWidth, "igGetColumnWidth");
    lib.bindSymbol(cast(void**)&igGetNavInputName, "igGetNavInputName");
    lib.bindSymbol(cast(void**)&igTableHeader, "igTableHeader");
    lib.bindSymbol(cast(void**)&igTabBarFindMostRecentlySelectedTabForActiveWindow, "igTabBarFindMostRecentlySelectedTabForActiveWindow");
    lib.bindSymbol(cast(void**)&ImGuiPayload_Clear, "ImGuiPayload_Clear");
    lib.bindSymbol(cast(void**)&ImGuiTextBuffer_reserve, "ImGuiTextBuffer_reserve");
    lib.bindSymbol(cast(void**)&ImDrawList__TryMergeDrawCmds, "ImDrawList__TryMergeDrawCmds");
    lib.bindSymbol(cast(void**)&ImGuiInputTextState_CursorAnimReset, "ImGuiInputTextState_CursorAnimReset");
    lib.bindSymbol(cast(void**)&ImRect_ClipWithFull, "ImRect_ClipWithFull");
    lib.bindSymbol(cast(void**)&igGetFontTexUvWhitePixel, "igGetFontTexUvWhitePixel");
    lib.bindSymbol(cast(void**)&igShowStackToolWindow, "igShowStackToolWindow");
    lib.bindSymbol(cast(void**)&ImDrawList_ChannelsSplit, "ImDrawList_ChannelsSplit");
    lib.bindSymbol(cast(void**)&igCalcWindowNextAutoFitSize, "igCalcWindowNextAutoFitSize");
    lib.bindSymbol(cast(void**)&igPopFont, "igPopFont");
    lib.bindSymbol(cast(void**)&igImTriangleContainsPoint, "igImTriangleContainsPoint");
    lib.bindSymbol(cast(void**)&igRenderRectFilledWithHole, "igRenderRectFilledWithHole");
    lib.bindSymbol(cast(void**)&igImFloor_Float, "igImFloor_Float");
    lib.bindSymbol(cast(void**)&igImFloor_Vec2, "igImFloor_Vec2");
    lib.bindSymbol(cast(void**)&ImDrawList_AddRect, "ImDrawList_AddRect");
    lib.bindSymbol(cast(void**)&igImParseFormatFindEnd, "igImParseFormatFindEnd");
    lib.bindSymbol(cast(void**)&ImGuiPlatformIO_destroy, "ImGuiPlatformIO_destroy");
    lib.bindSymbol(cast(void**)&ImGuiTableColumnSettings_destroy, "ImGuiTableColumnSettings_destroy");
    lib.bindSymbol(cast(void**)&ImGuiInputTextCallbackData_ClearSelection, "ImGuiInputTextCallbackData_ClearSelection");
    lib.bindSymbol(cast(void**)&igErrorCheckEndFrameRecover, "igErrorCheckEndFrameRecover");
    lib.bindSymbol(cast(void**)&ImGuiTextRange_split, "ImGuiTextRange_split");
    lib.bindSymbol(cast(void**)&ImBitVector_Clear, "ImBitVector_Clear");
    lib.bindSymbol(cast(void**)&igDockBuilderAddNode, "igDockBuilderAddNode");
    lib.bindSymbol(cast(void**)&igCreateNewWindowSettings, "igCreateNewWindowSettings");
    lib.bindSymbol(cast(void**)&igDockNodeGetWindowMenuButtonId, "igDockNodeGetWindowMenuButtonId");
    lib.bindSymbol(cast(void**)&ImGuiDockNode_IsRootNode, "ImGuiDockNode_IsRootNode");
    lib.bindSymbol(cast(void**)&igDockContextInitialize, "igDockContextInitialize");
    lib.bindSymbol(cast(void**)&igGetDrawListSharedData, "igGetDrawListSharedData");
    lib.bindSymbol(cast(void**)&igBeginChildEx, "igBeginChildEx");
    lib.bindSymbol(cast(void**)&ImGuiNextWindowData_ClearFlags, "ImGuiNextWindowData_ClearFlags");
    lib.bindSymbol(cast(void**)&igImFileClose, "igImFileClose");
    lib.bindSymbol(cast(void**)&ImFontGlyphRangesBuilder_GetBit, "ImFontGlyphRangesBuilder_GetBit");
    lib.bindSymbol(cast(void**)&igImRotate, "igImRotate");
    lib.bindSymbol(cast(void**)&igImGetDirQuadrantFromDelta, "igImGetDirQuadrantFromDelta");
    lib.bindSymbol(cast(void**)&igScrollToRectEx, "igScrollToRectEx");
    lib.bindSymbol(cast(void**)&igTableMergeDrawChannels, "igTableMergeDrawChannels");
    lib.bindSymbol(cast(void**)&ImFontAtlas_AddFont, "ImFontAtlas_AddFont");
    lib.bindSymbol(cast(void**)&igGetNavInputAmount2d, "igGetNavInputAmount2d");

    // Backends
    version (USE_GLFW) {
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_InstallCallbacks, "ImGui_ImplGlfw_InstallCallbacks");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_MonitorCallback, "ImGui_ImplGlfw_MonitorCallback");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_NewFrame, "ImGui_ImplGlfw_NewFrame");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_InitForOther, "ImGui_ImplGlfw_InitForOther");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_InitForVulkan, "ImGui_ImplGlfw_InitForVulkan");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_CharCallback, "ImGui_ImplGlfw_CharCallback");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_InitForOpenGL, "ImGui_ImplGlfw_InitForOpenGL");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_KeyCallback, "ImGui_ImplGlfw_KeyCallback");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_ScrollCallback, "ImGui_ImplGlfw_ScrollCallback");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_MouseButtonCallback, "ImGui_ImplGlfw_MouseButtonCallback");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_CursorPosCallback, "ImGui_ImplGlfw_CursorPosCallback");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_RestoreCallbacks, "ImGui_ImplGlfw_RestoreCallbacks");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_WindowFocusCallback, "ImGui_ImplGlfw_WindowFocusCallback");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_Shutdown, "ImGui_ImplGlfw_Shutdown");
        lib.bindSymbol(cast(void**)&ImGui_ImplGlfw_CursorEnterCallback, "ImGui_ImplGlfw_CursorEnterCallback");
    }

    version (USE_OpenGL3) {
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_DestroyFontsTexture, "ImGui_ImplOpenGL3_DestroyFontsTexture");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_CreateFontsTexture, "ImGui_ImplOpenGL3_CreateFontsTexture");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_CreateDeviceObjects, "ImGui_ImplOpenGL3_CreateDeviceObjects");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_Init, "ImGui_ImplOpenGL3_Init");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_DestroyDeviceObjects, "ImGui_ImplOpenGL3_DestroyDeviceObjects");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_NewFrame, "ImGui_ImplOpenGL3_NewFrame");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_Shutdown, "ImGui_ImplOpenGL3_Shutdown");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL3_RenderDrawData, "ImGui_ImplOpenGL3_RenderDrawData");
    }

    version (USE_SDL2) {
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_Shutdown, "ImGui_ImplSDL2_Shutdown");
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_InitForMetal, "ImGui_ImplSDL2_InitForMetal");
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_InitForOpenGL, "ImGui_ImplSDL2_InitForOpenGL");
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_InitForVulkan, "ImGui_ImplSDL2_InitForVulkan");
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_InitForD3D, "ImGui_ImplSDL2_InitForD3D");
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_ProcessEvent, "ImGui_ImplSDL2_ProcessEvent");
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_InitForSDLRenderer, "ImGui_ImplSDL2_InitForSDLRenderer");
        lib.bindSymbol(cast(void**)&ImGui_ImplSDL2_NewFrame, "ImGui_ImplSDL2_NewFrame");
    }

    version (USE_OpenGL2) {
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_CreateDeviceObjects, "ImGui_ImplOpenGL2_CreateDeviceObjects");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_Init, "ImGui_ImplOpenGL2_Init");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_DestroyDeviceObjects, "ImGui_ImplOpenGL2_DestroyDeviceObjects");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_NewFrame, "ImGui_ImplOpenGL2_NewFrame");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_RenderDrawData, "ImGui_ImplOpenGL2_RenderDrawData");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_CreateFontsTexture, "ImGui_ImplOpenGL2_CreateFontsTexture");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_Shutdown, "ImGui_ImplOpenGL2_Shutdown");
        lib.bindSymbol(cast(void**)&ImGui_ImplOpenGL2_DestroyFontsTexture, "ImGui_ImplOpenGL2_DestroyFontsTexture");
    }



    if (errorCount() != 0) return ImGuiSupport.badLibrary;

    return ImGuiSupport.ImGui_1_79;
}
